      PROGRAM POWRMETHOD
C     ----------------------------------------------------------------
C     Alg11'12.for   FORTRAN program for implementing Algorithm 11.1-2
C     
C     NUMERICAL METHODS: FORTRAN Programs, (c) John H. Mathews 1995
C     To accompany the text:
C     NUMERICAL METHODS for Math., Science & Engineering, 2nd Ed, 1992
C     Prentice Hall, Englewood Cliffs, New Jersey, 07632, U.S.A.
C     Prentice Hall, Inc.; USA, Canada, Mexico ISBN 0-13-624990-6
C     Prentice Hall, International Editions:   ISBN 0-13-625047-5
C     This free software is compliments of the author.
C     E-mail address:       in%"mathews@fullerton.edu"
C     
C     Algorithm 11.1 (Power Method).
C     Section   11.2, The Power Method, Page 557
C     
C     Algorithm 11.2 (Shifted Inverse Power Method).
C     Section   11.2, The Power Method, Page 558
C     ----------------------------------------------------------------
      PARAMETER(MaxR = 5)
      CHARACTER Ans*1,Ach*1,Mess*80
      INTEGER Count,CountR,CountS
      INTEGER I,InRC,Inum,N,P,Q
      REAL Alpha,Apq,C,Det,Epsilon
      REAL Err,Lambda,MaxA,S,Rnum,T
      INTEGER Row,MaxIter
      DIMENSION Row(1:MaxR)
      REAL X,XP,XQ,YP,YQ
      DIMENSION X(1:MaxR),XP(1:MaxR),XQ(1:MaxR)
      DIMENSION YP(1:MaxR),YQ(1:MaxR)
      REAL A,A1,V
      DIMENSION A(1:MaxR,1:MaxR),A1(1:MaxR,1:MaxR)
      DIMENSION V(1:MaxR,1:MaxR)
      INTEGER Meth,Power,ShiftedInverse
      INTEGER Proc,Auto,Observe
      EXTERNAL XMaxEle,DIST
      Auto = 0
      Observe = 1
      Power = 1
      ShiftedInverse = 2
      CALL MESSAGE(Meth)
      CALL INPUTS(A,A1,N,InRC,MaxIter)
      CALL PROCESSES(Proc)
      IF (Meth.EQ.Power) THEN
        CALL POWMETH(A,N,Epsilon,MaxIter,X,Lambda,Err,Count,Proc)
      ENDIF
      IF (Meth.EQ.ShiftedInverse) THEN
        WRITE(9,*)' '
        WRITE(9,*)'Enter the value Alpha = '
        READ(9,*) Alpha
        WRITE(9,*)' '
        CALL SHIFTEDINVPOWER(A,N,Epsilon,MaxIter,
     +Alpha,X,Lambda,Err,Count,Proc)
      ENDIF
      CALL PowrResults(A1,N,Epsilon,X,Lambda,Err,Count)
      END

      SUBROUTINE MATPRINT(A,N)
      PARAMETER(MaxR = 5)
      INTEGER Col,N,Row
      REAL A
      DIMENSION A(1:MaxR,1:MaxR)
      DO Row=1,N
        WRITE(9,999) ( A(Row,Col), Col=1,N )
      ENDDO
      RETURN
999   FORMAT(1X,5F15.7,4X)
      END

      SUBROUTINE Voutput(X,N)
      PARAMETER(MaxR = 5)
      INTEGER N,Row
      REAL X
      DIMENSION X(1:MaxR)
      WRITE(9,*)
      DO Row=1,N
        WRITE(9,*) 'X(',Row,')  =  ',X(Row)
      ENDDO
      RETURN
      END

      REAL FUNCTION XMaxEle(X,N)
      PARAMETER(MaxR = 5)
      REAL X,MaxE
      DIMENSION X(1:MaxR)
      INTEGER J,N
      MaxE = 0
      DO J=1,N
        IF ABS(X(J)) > Abs(MaxE) THEN
          MaxE = X(J)
        ENDIF
      ENDDO
      XMaxEle = MaxE
      RETURN
      END

      REAL FUNCTION DIST(X,Y,N)
      PARAMETER(MaxR = 5)
      REAL X,Y,Sum
      DIMENSION X(1:MaxR),Y(1:MaxR)
      INTEGER J,N
      Sum = 0
      DO J=1,N
        Sum = Sum + (Y(J) - X(J))*(Y(J) - X(J))
      ENDDO
      DIST = SQRT(Sum)
      RETURN
      END
      
      SUBROUTINE POWMETH(A,N,Epsilon,MaxIter,X,Lambda,Err,Count,Proc)
      PARAMETER(MaxR = 5)
      REAL A,X,Y,Epsilon,Lambda,Err,C1,DC,DV,Sum
      DIMENSION X(1:MaxR),Y(1:MaxR)
      DIMENSION A(1:MaxR,1:MaxR)
      INTEGER Count,I,J,N,MaxIter
      INTEGER State,Done,Iterating,Proc,Observe
      EXTERNAL XMaxEle,DIST
      DO J=1,N
        X(J) = 1
      ENDDO
      Lambda = 0
      Count = 0
      Err = 1
      Done = 0
      Iterating = 1
      State = Iterating
      Observe = 1
      WHILE ((Count.LE.MaxIter).AND.(State.EQ.Iterating))
        DO I=1,N                             !Matrix multiplication Y = AX}
            Sum = 0
            DO J=1,N
              Sum = Sum + A(I,J)*X(J)
            ENDDO
            Y(I) = Sum
        ENDDO
        C1 = 0
        C1 = XMaxEle(Y,N)                    !Find largest element of Y
        DC = ABS(Lambda - C1)
        DO J=1,N                             !Perform scalar multiplication
          Y(J) = (1/C1)*Y(J)
        ENDDO
        DV = DIST(X,Y,N)
        Err = MAX(DC,DV)
        DO J=1,N                             !Update vector  X
          X(J) = Y(J)
        ENDDO
        Lambda = C1                          !Update scalar Lambda
        State = Done
        IF (Err.GT.Epsilon) THEN             !Check for convergence
          State = Iterating
        ENDIF
        Count = Count+1
        IF (Proc.EQ.Observe) THEN
            WRITE(9,*)' '
            WRITE(9,*)'Lambda = ', Lambda
            CALL Voutput(X,N)
        ENDIF
      REPEAT
      RETURN                               !OUTPUT vector X and scalar Lambda
      END

      SUBROUTINE FACTOR(A,Row,N,Det)
      PARAMETER(MaxR = 5)
      INTEGER Row,N
      DIMENSION Row(1:MaxR)
      REAL A,Det
      DIMENSION A(1:MaxR,1:MaxR)
      INTEGER C,J,K,P,RowK,RowP,T
        Det = 1
        DO J=1,N                           !Initialize Pointer Vector
          Row(J) = J
        ENDDO
        DO P=1,(N-1)                       !Upper Triangularization Loop
            DO K=(P+1),N
                IF ABS(A(Row(K),P)) > ABS(A(Row(P),P)) THEN
                  T = Row(P)
                  Row(P) = Row(K)
                  Row(K) = T
                  Det = -Det
                ENDIF
            ENDDO
            Det = Det*A(Row(P),P)
            IF (Det.EQ.0) RETURN                   !Check Singular Matrix
            DO K=(P+1),N                           !Gaussian Elimination
              RowK = Row(K)
              RowP = Row(P)
              A(RowK,P) = A(RowK,P)/A(RowP,P)
              DO C=(P+1),N
                A(RowK,C) = A(RowK,C) - A(RowK,P)*A(RowP,C)
        ENDDO
            ENDDO
        ENDDO
      Det = Det*A(Row(N),N)
      RETURN
      END

      SUBROUTINE SOLVE(A,Row,B,X,N)
      PARAMETER(MaxR = 5)
      INTEGER Row,N
      DIMENSION Row(1:MaxR)
      REAL A,B,X
      DIMENSION B(1:MaxR),X(1:MaxR)
      DIMENSION A(1:MaxR,1:MaxR)
      INTEGER C,K,RowK
      REAL Sum
      DO K=1,N
        IF (A(Row(K),K).EQ.0) RETURN               !Check Singular Matrix
      ENDDO
      X(1) = B(Row(1))                             !Forward Substitution
      DO K=2,N
          Sum = 0;
          RowK = Row(K)
          DO C=1,(K-1)
            Sum = Sum + A(RowK,C)*X(C)
          ENDDO
          X(K) = B(RowK) - Sum                   !End Forward Substitution
      ENDDO
      X(N) = X(N)/A(Row(N),N)                    !Back Substitution
      DO K=(N-1),1,-1
          Sum = 0
          RowK = Row(K)
          DO C=(K+1),N
            Sum = Sum + A(RowK,C)*X(C)
          ENDDO
          X(K) = (X(K) - Sum)/A(RowK,K)
      ENDDO
      RETURN
      END

      SUBROUTINE SHIFTEDINVPOWER(A,N,Epsilon,MaxIter,Alpha,X,
     +Lambda,Err,Count,Proc)
      PARAMETER(MaxR = 5)
      REAL A,A1,X,Y
      DIMENSION X(1:MaxR),Y(1:MaxR)
      DIMENSION A(1:MaxR,1:MaxR),A1(1:MaxR,1:MaxR)
      INTEGER Row,Count,J,K,N,MaxIter
      DIMENSION Row(1:MaxR)
      INTEGER State,Done,Iterating,Proc,Observe
      REAL Epsilon,Alpha,Lambda,Err
      REAL C1,DC,DV,Det
      EXTERNAL XMaxEle,DIST
      Done = 0
      Iterating = 1
      Observe = 1
      DO J=1,N                                !Initialize the matrix V
        X(J) = 1
      ENDDO
      DO J=1,N                                !Form the matrix A - aIpha I
        DO K=1,N
          A1(J,K) = A(J,K)
        ENDDO
      ENDDO
      DO J=1,N
        A1(J,J) = A1(J,J) - Alpha
      ENDDO
      Lambda = 0
      Count = 0
      Err = 1
      CALL FACTOR(A1,Row,N,Det)
      State = Iterating
      WHILE ((Count.LE.MaxIter).AND.(State.EQ.Iterating))
        CALL SOLVE(A1,Row,X,Y,N)          !Solve system A1 Y = X
        C1 = XMaxEle(Y,N)                 !Find largest element of Y
        DC = ABS(Lambda - C1)
        DO J=1,N                          !Perform scalar multiplication
          Y(J) = (1/C1)*Y(J)
        ENDDO
        DV = DIST(X,Y,N);
        Err = MAX(DC,DV)
        DO J=1,N                          !Update vector  X
          X(J) = Y(J)
        ENDDO
        Lambda = C1                       !Update scalar Lambda 
        State = Done
        IF (Err.GT.Epsilon) THEN          !Check for convergence
          State = Iterating
        ENDIF
        Count = Count+1
        IF (Proc.EQ.Observe) THEN
          WRITE(9,*)' '
          Lambda = Alpha+1/C1
          WRITE(9,*)'Alpha = ',Alpha, '    ','C',Count,'  =  ',C1
          WRITE(9,*)'Lambda = ', Lambda
          CALL Voutput(X,N);
        ENDIF
      REPEAT
      Lambda = Alpha+1/C1                 !OUTPUT vector X and scalar Lambda
      RETURN
      END

      SUBROUTINE MATIN(Ach,A,N,InRC)
      PARAMETER(MaxR = 5)
      INTEGER Col,InRC,N,Row
      REAL A
      CHARACTER Ach*1
      DIMENSION A(1:MaxR,1:MaxR)
      DO 20 Row=1,N
      DO 10 Col=1,N
        A(Row,Col)=0
10    CONTINUE
20    CONTINUE
      WRITE(9,*)'GIVE THE ELEMENTS OF THE MATRIX ',Ach
      WRITE(9,*)' '
      IF (InRC.EQ.0) THEN
        DO 40 Row=1,N
          WRITE(9,*)'GIVE THE ELEMENTS OF ROW ',Row
          WRITE(9,*)' '
        DO 30 Col=1,N
          WRITE(9,*) Ach,'(',Row,',',Col,') = '
          READ(9,*) A(Row,Col)
30      CONTINUE
40      CONTINUE
      ELSE
        DO 60 Col=1,N
          WRITE(9,*)'GIVE THE ELEMENTS OF COLUMN ',Col
          WRITE(9,*)' '
        DO 50 Row=1,N
          WRITE(9,*) Ach,'(',Row,',',Col,') = '
          READ(9,*) A(Row,Col)
50      CONTINUE
60      CONTINUE
      ENDIF
      RETURN
      END
       
      SUBROUTINE PowrResults(A1,N,Epsilon,X,Lambda,Err,Count)
      PARAMETER(MaxR = 5)
      REAL A1,X
      DIMENSION X(1:MaxR)
      DIMENSION A1(1:MaxR,1:MaxR)
      INTEGER C,R,N,Count
      REAL A0,Epsilon,Lambda,Err
      WRITE(9,*)' '
      WRITE(9,*)'The matrix  A  is:'
      WRITE(9,*)' '
      DO 10 R=1,N
        WRITE(9,999) ( A1(R,C), C=1,N )
      ENDDO
      WRITE(9,*)' '
      WRITE(9,*)'The dominant eigenvalue of  A  is:'
      WRITE(9,*)' '
      WRITE(9,*)'Lambda = ', Lambda 
      WRITE(9,*)' '
      WRITE(9,*)' '
      WRITE(9,*)'The dominant eigenvector of  A  is:'
      WRITE(9,*)' '
      CALL Voutput(X,N)
      WRITE(9,*)' '
      WRITE(9,*)' '
      IF (Count.EQ.Max) THEN
        WRITE(9,*)'The maximum number of iterations was used.'
        WRITE(9,*)' '
      ENDIF
      PAUSE
      RETURN
999   FORMAT(1X,5F15.7,4X)
      END
      
      SUBROUTINE CHOOSEMETH(Meth)
      INTEGER I,Meth,Power,ShiftedInverse
      Power = 1
      ShiftedInverse = 2
      WRITE(9,*)' '
      WRITE(9,*)'Choose a strategy for finding an eigen-pair.'
      WRITE(9,*)' '
      WRITE(9,*)'<1> The power method.'
      WRITE(9,*)' '
      WRITE(9,*)'<2> The shifted inverse power method.'
      WRITE(9,*)' '
      WRITE(9,*)'    SELECT the strategy  <1 or 2> ?  '
      READ(9,*) I
      IF ((I.LT.1).OR.(2.LT.I)) I = 1
      IF (I.EQ.1) Meth = Power
      IF (I.EQ.2) Meth = ShiftedInverse
      RETURN
      END
      
      SUBROUTINE MESSAGE(Meth)
      INTEGER Meth
      WRITE(9,*)'     POWER METHOD FOR FINDING AN EIGEN-PAIR'
      WRITE(9,*)' '
      WRITE(9,*)'Assume that A is an N by N real matrix and that it'
      WRITE(9,*)' '
      WRITE(9,*)'has a full set of eigenvectors  V , V  ,..., V . '
      WRITE(9,*)'                                 1   2        N '
      WRITE(9,*)' '
      WRITE(9,*)'The powermethod of iteration is used to find an'
      WRITE(9,*)' '
      WRITE(9,*)'eigenvalue and its corresponding eigenvector.'
      WRITE(9,*)' '
      WRITE(9,*)' '
      WRITE(9,*)'     Press the  <ENTER>  key.  '
      PAUSE
      CALL CHOOSEMETH(Meth)
      RETURN
      END

      SUBROUTINE INPUTS(A,A1,N,InRC,MaxIter)
      PARAMETER(MaxR = 5)
      REAL A,A1
      DIMENSION A(1:MaxR,1:MaxR),A1(1:MaxR,1:MaxR)
      INTEGER N,InRC,MaxIter,Row,Col
      WRITE(9,*)' '
      WRITE(9,*)'We will now proceed with the power method'
      WRITE(9,*)' '
      WRITE(9,*)'A  must be a matrix of dimension  N by N.'
      WRITE(9,*)' '
      WRITE(9,*)'{N  must be an integer between 2 and 10}'
      WRITE(9,*)' '
      WRITE(9,*)'ENTER  N  = '
      READ(9,*) N
      IF (N.LT.2)  N = 2
      IF (N.GT.10) N = 10
      WRITE(9,*)'Iteration will continue until each coordinate'
      WRITE(9,*)' '
      WRITE(9,*)'of the eigenvector has converged with an '
      WRITE(9,*)' '
      WRITE(9,*)'error less than  Epsilon  or  the maximum'
      WRITE(9,*)' '
      WRITE(9,*)'number of iterations  Max  has been reached.'
      WRITE(9,*)' '
      WRITE(9,*)'ENTER  Epsilon = '
      READ(9,*) Epsilon
      IF (Epsilon.LT. 0.000000001) Epsilon = 0.000000001
      WRITE(9,*)' '
      WRITE(9,*)'ENTER  Max = '
      READ(9,*) MaxIter
      IF (50.LT.MaxIter) MaxIter = 50
      InRC = 0
      WRITE(9,*)'WANT TO INPUT COLUMNS ? <Y/N> '
      READ(9,'(A)') Ans
      IF (Ans.EQ.'Y' .OR. Ans.EQ.'y') THEN
        InRC = 1
      ENDIF
      WRITE(9,*)' '
      WRITE(9,*)'ENTER THE COEFFICIENT MATRIX  A:'
      CALL MATIN('A',A,N,InRC)
      DO Row=1,N
        DO Col=1,N
          A1(Row,Col)=A(Row,Col)
        ENDDO
      ENDDO
      RETURN
      END

      SUBROUTINE PROCESSES(Proc)
      INTEGER I,Proc,Auto,Observe
      Auto = 0
      Observe = 1
      WRITE(9,*)' '
      WRITE(9,*)'To what extent do you want to control the program?'
      WRITE(9,*)' '
      WRITE(9,*)'<1> The computer does it all automatically.'
      WRITE(9,*)' '
      WRITE(9,*)'<2> Computer selects, but we observe each step.'
      WRITE(9,*)' '
      WRITE(9,*)'SELECT your choice for input  <1 - 2> ? '
      READ(9,*) I
      IF ((I.LT.1).OR.(2.LT.I)) I = 2
      IF (I.EQ.1) Proc = Auto
      IF (I.EQ.2) Proc = Observe
      RETURN
      END
