      PROGRAM FALSEPOS
C     ----------------------------------------------------------------
C     Alg2'3.for   FORTRAN program for implementing Algorithm 2.3
C     
C     NUMERICAL METHODS: FORTRAN Programs, (c) John H. Mathews 1995
C     To accompany the text:
C     NUMERICAL METHODS for Math., Science & Engineering, 2nd Ed, 1992
C     Prentice Hall, Englewood Cliffs, New Jersey, 07632, U.S.A.
C     Prentice Hall, Inc.; USA, Canada, Mexico ISBN 0-13-624990-6
C     Prentice Hall, International Editions:   ISBN 0-13-625047-5
C     This free software is compliments of the author.
C     E-mail address:       in%"mathews@fullerton.edu"
C     
C     Algorithm 2.3 (False position or Regula Falsi Method).
C     Section   2.2, Bracketing Methods for Locating a Root, Page  62
C     ----------------------------------------------------------------
      PARAMETER(Delta=5E-6,Epsilon=5E-6,Max=99)
      INTEGER Cond,K
      REAL A,A1,B,B1,C,DX,YC
      CHARACTER ANS*1
      EXTERNAL F
10    CALL GETPTS(A,B)
      A1=A
      B1=B
      CALL REGULA(F,A1,B1,Delta,Epsilon,Max,C,YC,D,K,Cond)
      CALL RESULT(F,A,B,Delta,Epsilon,Max,C,YC,D,K,Cond)
      WRITE(9,*)' '
      WRITE(9,*)'WANT TO TRY ANOTHER INTERVAL? <Y/N> '
      READ(9,'(A)') ANS
      IF(ANS .EQ. 'Y' .OR. ANS .EQ. 'y') GOTO 10
      STOP
      END

      REAL FUNCTION F(X)
        F = SIN(X)/COS(X)
      RETURN
      END

      SUBROUTINE PRINTFN
        WRITE(9,*)'F(X) = SIN(X)/COS(X)'
      RETURN
      END
      
      SUBROUTINE REGULA(F,A,B,Delta,Epsilon,Max,C,YC,D,K,Cond)
      PARAMETER(Big=1E3)
      INTEGER Cond,K,Max
      REAL A,B,C,D,Delta,DX,Epsilon,YC
      EXTERNAL F
      K=0
      YA=F(A)
      YB=F(B)
      D=B-A
      Cond=0
      IF (YA*YB.GE.0) THEN
        Cond=1
        RETURN
      ENDIF
      WHILE ((Cond.EQ.0).AND.(K.LT.Max))
        DXA=YA*(B-A)/(YB-YA)
        DXB=YB*(B-A)/(YB-YA)
        IF ABS(DXA).LT.ABS(DXB) THEN
          C=A-DXA
          D=ABS(DXA)
        ELSE
          C=B-DXB
          D=ABS(DXB)
        ENDIF
        YC=F(C)
        IF (D.LT.Delta) Cond=3
        IF (ABS(YC).LT.Epsilon) THEN
          A=C
          B=C
          Cond=4
        ELSEIF ((YB*YC).GT.0) THEN
          B=C
          YB=YC
        ELSE
          A=C
          YA=YC
        ENDIF
        K=K+1
        WRITE(9,1000) K,A,C,B
      REPEAT
      IF (YC.EQ.0) Cond=2
      IF (D.LT.Delta).AND.(Cond.NE.2) Cond=3
      IF (Cond.EQ.3).AND.(ABS(YC).LT.Epsilon)  Cond=5
      IF (ABS(YA).GT.Big).AND.(ABS(YB).GT.Big) Cond=6
      IF (Cond.EQ.0).AND.(ABS(YC).LT.Epsilon)  Cond=4
      PAUSE
      RETURN
1000  FORMAT(I2,4X,F15.7,4X,F15.7,4X,F15.7)
      END

      SUBROUTINE XEGULA(F,A,B,Delta,Epsilon,Max,C,YC,D,K,Cond)
      PARAMETER(Big=1E3)
      INTEGER Cond,K,Max
      REAL A,B,C,D,Delta,DX,Epsilon,YC
      EXTERNAL F
      K=0
      YA=F(A)
      YB=F(B)
      D=B-A
      Cond=0
      IF (YA*YB.GE.0) THEN
        Cond=1
        RETURN
      ENDIF
10    IF ((Cond.EQ.0).AND.(K.LT.Max)) THEN
        DXA=YA*(B-A)/(YB-YA)
        DXB=YB*(B-A)/(YB-YA)
        IF ABS(DXA).LT.ABS(DXB) THEN
          C=A-DXA
          D=ABS(DXA)
        ELSE
          C=B-DXB
          D=ABS(DXB)
        ENDIF
        YC=F(C)
        IF (D.LT.Delta) Cond=3
        IF (ABS(YC).LT.Epsilon) THEN
          A=C
          B=C
          Cond=4
        ELSEIF ((YB*YC).GT.0) THEN
          B=C
          YB=YC
        ELSE
          A=C
          YA=YC
        ENDIF
        K=K+1
        WRITE(9,1000) K,A,C,B,D
        GOTO 10
      ENDIF
      IF (YC.EQ.0) Cond=2
      IF (D.LT.Delta) THEN
        IF (Cond.NE.2) Cond=3
        IF ((Cond.EQ.3).AND.(ABS(YC).LT.Epsilon))  Cond=5
        IF ((ABS(YA).GT.Big).AND.(ABS(YB).GT.Big)) Cond=6
      ENDIF
      IF ((Cond.EQ.0).AND.(ABS(YC).LT.Epsilon))  Cond=4
      PAUSE
      RETURN
1000  FORMAT(I2,4X,F15.7,4X,F15.7,4X,F15.7,4X,F15.7)
      END

      SUBROUTINE GETPTS(A,B)
      INTEGER I
      REAL A,B
      DO 10 I=1,17
        WRITE(9,*)' '
10    CONTINUE
      WRITE(9,*)'THE FALSE POSITION METHOD WILL BE USED'
      WRITE(9,*)'TO FIND A ZERO THE FUNCTION'
      WRITE(9,*)' '
      CALL PRINTFN
      WRITE(9,*)' '
      WRITE(9,*)'THAT LIES IN THE INTERVAL  [A,B]'
      WRITE(9,*)' '
      WRITE(9,*)'ENTER LEFT  ENDPOINT A = '
      READ(9,*) A
      WRITE(9,*)'ENTER RIGHT ENDPOINT B = '
      READ(9,*) B
      WRITE(9,*)' '
      RETURN
      END

      SUBROUTINE RESULT(F,A,B,Delta,Epsilon,Max,C,YC,D,K,Cond)
      INTEGER Cond,I,J,K,Max
      REAL A,B,C,Delta,D,Epsilon,YC
      EXTERNAL F
      DO 10 I=1,16
        WRITE(9,*)' '
10    CONTINUE
      WRITE(9,*)'THE FALSE POSITION METHOD WAS USED TO FIND A ZERO'
      WRITE(9,*)'OF THE FUNCTION'
      WRITE(9,*)' '
      CALL PRINTFN
      WRITE(9,*)' '
      WRITE(9,*)'IN THE INTERVAL  [',A,'  ,',B,'  ]'
      WRITE(9,*)' '
      IF (Cond.EQ.1) THEN
        WRITE(9,*)'THE VALUES F(A) AND F(B)'
        WRITE(9,*)'DO NOT DIFFER IN SIGN.'
        WRITE(9,*)' '
        WRITE(9,*)'F(',A,'  ) =',F(A)
        WRITE(9,*)'F(',B,'  ) =',F(B)
        WRITE(9,*)' '
        WRITE(9,*)'THE FALSE POSITION METHOD CANNOT BE USED.'
      ELSEIF ((Cond.GE.2).AND.(Cond.LE.5)) THEN
        WRITE(9,*)'THE FALSE POSITION METHOD TOOK ',K,' ITERATIONS.'
        WRITE(9,*)' '
        WRITE(9,*)'  THE ROOT IS  C =',C
        WRITE(9,*)' '
        WRITE(9,*)'THE ACCURACY IS +-',D
        WRITE(9,*)' '
        WRITE(9,*)'FUNCTION VALUE  F(',C,'  ) =',F(C)
        WRITE(9,*)' '
        IF (Cond.EQ.2) THEN
          WRITE(9,*)'THE FUNCTION VALUE IS EXACTLY ZERO!'
        ENDIF
        IF ((Cond.EQ.3).OR.(Cond.EQ.5)) THEN
          WRITE(9,*)'THE ABSCISSA IS WITHIN TOLERANCE'
        ENDIF
        IF ((Cond.EQ.4).OR.(Cond.EQ.5)) THEN
          WRITE(9,*)'THE FUNCTION VALUE IS IS WITHIN TOLERANCE'
        ENDIF
      ELSEIF ((Cond.EQ.0).OR.(Cond.EQ.6)) THEN
        WRITE(9,*)'THE CURRENT ITERATE IS C(',K,') =',C
        WRITE(9,*)' '
        WRITE(9,*)'CONSECUTIVE ITERATES DIFFER BY',D
        WRITE(9,*)' '
        WRITE(9,*)'THE CURRENT FUNCTION VALUE  F(',C,'  ) =',F(C)
        WRITE(9,*)' '
        IF (Cond.EQ.0) THEN
          WRITE(9,*)'THE MAXIMUM NUMBER OF ITERATIONS WAS EXCEEDED.'
        ENDIF
        IF (Cond.EQ.6) THEN
          WRITE(9,*)'SURPRISE, A POLE WAS FOUND INSTEAD OF A ZERO!'
        ENDIF
      ENDIF
      WRITE(9,*)' '
      RETURN
      END
