      PROGRAM APPROXLO
C     ----------------------------------------------------------------
C     Alg2'4.for   FORTRAN program for implementing Algorithm 2.4
C     
C     NUMERICAL METHODS: FORTRAN Programs, (c) John H. Mathews 1995
C     To accompany the text:
C     NUMERICAL METHODS for Math., Science & Engineering, 2nd Ed, 1992
C     Prentice Hall, Englewood Cliffs, New Jersey, 07632, U.S.A.
C     Prentice Hall, Inc.; USA, Canada, Mexico ISBN 0-13-624990-6
C     Prentice Hall, International Editions:   ISBN 0-13-625047-5
C     This free software is compliments of the author.
C     E-mail address:       in%"mathews@fullerton.edu"
C     
C     Algorithm 2.4 (Approximate Location of Roots).
C     Section   2.3, Initial Approx. and Convergence Criteria, Page 70
C     ----------------------------------------------------------------
      PARAMETER(Epsilon=0.05,MN=201,MR=50)
      INTEGER N,M
      REAL A,B,R,X,Y
      CHARACTER ANS*1
      DIMENSION X(0:MN),R(0:MR),Y(0:MN)
      EXTERNAL F
10    CALL INPUT(A,B,N)
      CALL LOCATE(F,A,B,N,Epsilon,R,M)
      CALL RESULTS(F,A,B,N,R,M)
      WRITE(9,*)' '
      WRITE(9,*) 'WANT TO TRY ANOTHER INTERVAL? <Y/N> '
      READ(9,'(A)') ANS
      IF (ANS.EQ.'Y'.OR.ANS.EQ.'y') GOTO 10
      STOP
      END

      REAL FUNCTION F(X)
      REAL X
        F=SIN(X)+1
      RETURN
      END

      SUBROUTINE PRINTFN
        WRITE(9,*)'F(X)  =  SIN(X)+1'
      RETURN
      END

      SUBROUTINE LOCATE(F,A,B,N,Epsilon,R,M)
      PARAMETER(MN=201,MR=50)
      INTEGER K,M,N
      REAL A,B,Epsilon,H,MaxY,MinY,R,Slope,Small,X,Y
      DIMENSION X(0:MN),R(0:MR),Y(0:MN)
      EXTERNAL F
      H=(B-A)/N
      DO K=0,N
          X(K)=A+H*K
          Y(K)=F(X(K))
      ENDDO
      MaxY=Y(0)
      DO K=1,N
        IF (MaxY.LT.Y(K)) MaxY=Y(K)
      ENDDO
      MinY=Y(0)
      DO K=1,N
        IF (MinY.GT.Y(K)) MinY=Y(K)
      ENDDO
      Small=(MaxY-MinY)*Epsilon
      M=0
      Y(N+1)=Y(N)
      DO K=1,N
        IF ((Y(K-1)*Y(K)).LE.0) THEN
          M=M+1
          R(M)=(X(K-1)+X(K))/2
        ENDIF
        Slope=(Y(K)-Y(K-1))*(Y(K+1)-Y(K))
        IF ((ABS(Y(K)).LT.Small).AND.(Slope.LT.0)) THEN
          M=M+1
          R(M)=X(K)
        ENDIF
      ENDDO
      RETURN
      END

      SUBROUTINE XOCATE(F,A,B,N,Epsilon,R,M)
      PARAMETER(MN=201,MR=50)
      INTEGER K,M,N
      REAL A,B,Epsilon,H,MaxY,MinY,R,Slope,Small,X,Y
      DIMENSION X(0:MN),R(0:MR),Y(0:MN)
      EXTERNAL F
      H=(B-A)/N
      DO 10 K=0,N
          X(K)=A+H*K
          Y(K)=F(X(K))
10    CONTINUE
      MaxY=Y(0)
      DO 20 K=1,N
        IF (MaxY.LT.Y(K)) THEN
          MaxY=Y(K)
        ENDIF
20    CONTINUE
      MinY=Y(0)
      DO 30 K=1,N
        IF (MinY.GT.Y(K)) THEN
          MinY=Y(K)
        ENDIF
30    CONTINUE
      Small=(MaxY-MinY)*Epsilon
      M=0
      Y(N+1)=Y(N)
      DO 40 K=1,N
        IF ((Y(K-1)*Y(K)).LE.0) THEN
          M=M+1
          R(M)=(X(K-1)+X(K))/2
        ENDIF
        Slope=(Y(K)-Y(K-1))*(Y(K+1)-Y(K))
        IF ((ABS(Y(K)).LT.Small).AND.(Slope.LT.0)) THEN
          M=M+1
          R(M)=X(K)
        ENDIF
40    CONTINUE
      RETURN
      END

      SUBROUTINE INPUT(A,B,N)
      INTEGER I,N
      REAL A,B
      DO 10 I=1,18
      WRITE(9,*)' '
10    CONTINUE
      WRITE(9,*)'APPROXIMATE LOCATIONS FOR THE ZEROS OF THE FUNCTION'
      WRITE(9,*)' '
      CALL PRINTFN
      WRITE(9,*)' '
      WRITE(9,*)'THE INTERVAL [A,B] IS DIVIDED INTO N SUBINTERVALS'
      WRITE(9,*)' '
      WRITE(9,*)'AND A SEARCH IS PERFORMED TO DETERMINE IF  F(X)'
      WRITE(9,*)' '
      WRITE(9,*)'CHANGES SIGN OR IS CLOSE TO ZERO ON EACH SUBINTERVAL.'
      WRITE(9,*)' '
      WRITE(9,*)'ENTER  LEFT  ENDPOINT  A = '
      READ(9,*) A
      WRITE(9,*)' '
      WRITE(9,*)'ENTER  RIGHT ENDPOINT  B = '
      READ(9,*) B
      WRITE(9,*)' '
      WRITE(9,*)'NUMBER OF SUBINTERVALS N = '
      READ(9,*) N
      RETURN
      END

      SUBROUTINE RESULTS(F,A,B,N,R,M)
      PARAMETER(MR=50)
      INTEGER I,K,M,N
      REAL A,B,R
      DIMENSION R(0:MR)
      EXTERNAL F
      DO 10 I=1,17
        WRITE(9,*)' '
10    CONTINUE
      H=(B-A)/N
      WRITE(9,*)'WE SEARCHED TO FIND APPROXIMATE LOCATIONS FOR THE ZEROS
     + OF'
      WRITE(9,*)' '
      CALL PRINTFN
      WRITE(9,*)' '
      WRITE(9,*)'IN THE INTERVAL [',A,' ,',B,'  ].'
      WRITE(9,*)' '
      WRITE(9,*)'THE SEARCH WAS PERFORMED USING ',N,' SUBINTERVALS'
      WRITE(9,*)' '
      WRITE(9,*)'OF EQUAL WIDTH   H =',H
      WRITE(9,*)' '
      WRITE(9,*)' '
      IF (M.GE.0) THEN
        WRITE(9,*)'WE FOUND ',M,'  APPROXIMATE LOCATION(S).'
        WRITE(9,*)' '
        WRITE(9,*)'THEIR ABSCISSAS X(K) AND ORDINATES Y(K) ARE:'
        WRITE(9,*)' '
        DO 20 K=1,M
          WRITE(9,*)'X(',K,') =',R(K),'       Y(',K,') =',F(R(K))
20      CONTINUE
      ELSE
        WRITE(9,*)'NO APPROXIMATE LOCATIONS FOR ZEROS OF F(X) WERE FOUND
     +.'
        WRITE(9,*)' '
        WRITE(9,*)'YOU COULD TRY ANOTHER SEARCH USING A SMALLER STEP H,'
        WRITE(9,*)' '
        WRITE(9,*)'OR YOU COULD TRY A SEARCH IS DIFFERENT INTERVAL [A,B]
     +.'
      ENDIF
      RETURN
      END
