      PROGRAM NLSEIDEL
C     ----------------------------------------------------------------
C     Alg2'9.for   FORTRAN program for implementing Algorithm 2.9
C     
C     NUMERICAL METHODS: FORTRAN Programs, (c) John H. Mathews 1995
C     To accompany the text:
C     NUMERICAL METHODS for Math., Science & Engineering, 2nd Ed, 1992
C     Prentice Hall, Englewood Cliffs, New Jersey, 07632, U.S.A.
C     Prentice Hall, Inc.; USA, Canada, Mexico ISBN 0-13-624990-6
C     Prentice Hall, International Editions:   ISBN 0-13-625047-5
C     This free software is compliments of the author.
C     E-mail address:       in%"mathews@fullerton.edu"
C     
C     Algorithm 2.9 (Nonlinear Seidel Iteration).
C     Section   2.6, Iteration for Nonlinear Systems, Page 108
C     ----------------------------------------------------------------
      PARAMETER(Max=199,Tol=1E-6)
      INTEGER Count
      REAL P0,Q0,Pold,Qold,Pnew,Qnew,Sep
      CHARACTER Ans*1
      EXTERNAL G1,G2
10    CALL INPUTS(P0,Q0,Pold,Qold)
      CALL ITERATE(G1,G2,Pold,Qold,Pnew,Qnew,Tol,Max,Sep,Count)
      CALL RESULTS(P0,Q0,Pold,Qold,Pnew,Qnew,Tol,Sep,Count)
      WRITE(9,*)' '
      WRITE(9,*)'Want to try a different starting point ? <Y/N> '
      READ(9,'(A)') Ans
      IF (Ans.EQ.'Y' .OR. Ans.EQ.'y') GOTO 10
      END

      REAL FUNCTION G1(X,Y)
      REAL X,Y
        G1=(X*X - Y + 0.5)/2
      RETURN
      END

      REAL FUNCTION G2(X,Y)
      REAL X,Y
       G2=(- X*X - 4*Y*Y + 8*Y + 4)/8
      RETURN
      END

      SUBROUTINE PRINTFUN
      WRITE(9,*)' '
      WRITE(9,*)'     G (X,Y) = (X*X - Y + 0.5)/2'
      WRITE(9,*)'      1'
      WRITE(9,*)' '
      WRITE(9,*)'     G (X,Y) = (- X*X - 4*Y*Y + 8*Y + 4)/8'
      WRITE(9,*)'      2'
      WRITE(9,*)' '
      RETURN
      END

      SUBROUTINE ITERATE(G1,G2,Pold,Qold,Pnew,Qnew,Tol,Max,Sep,Count)
      PARAMETER(Big=1E9)
      INTEGER Count,K,Max
      REAL P0,Pnew,Pold,Q0,Qnew,Qold,Sep,Tol
      EXTERNAL G1,G2
      Sep=1
      K=0
      Pnew=Pold
      Qnew=Qold
      WHILE (K.LT.Max).AND.(Sep.GT.Tol).AND.
     &      (ABS(Pnew).LT.Big).AND.(ABS(Qnew).LT.Big)
        Pold=Pnew
        Qold=Qnew
        K=K+1
        Pnew=G1(Pold,Qold)
        Qnew=G2(Pnew,Qold)
        Sep=ABS(Pnew-Pold)+ABS(Qnew-Qold)
        WRITE(9,1000) K,Pnew,Qnew
      REPEAT
      Count=K
1000  FORMAT(I2,4X,5F18.7)
      PAUSE
      RETURN
      END

      SUBROUTINE XTERATE(G1,G2,Pold,Qold,Pnew,Qnew,Tol,Max,Sep,Count)
      PARAMETER(Big=1E10)
      INTEGER Count,K,Max
      REAL P0,Pnew,Pold,Q0,Qnew,Qold,Sep,Tol
      EXTERNAL G1,G2
      Sep=1
      K=0
      Pnew=Pold
      Qnew=Qold
      WRITE(9,999) K,Pnew,Qold
10    IF (K.LT.Max.AND.Sep.GT.Tol) THEN
      Pold=Pnew
      Qold=Qnew
      K=K+1
      Pnew=G1(Pold,Qold)
      Qnew=G2(Pnew,Qold)
      Sep=ABS(Pnew-Pold)+ABS(Qnew-Qold)
      WRITE(9,999) K,Pnew,Qold
      IF (ABS(Pnew).GT.Big).OR.(ABS(Qnew).GT.Big) GOTO 20
      GOTO 10
      ENDIF
20    CONTINUE
      Count=K
999   FORMAT(I2,4X,5F18.7)
      PAUSE
      RETURN
      END

      SUBROUTINE INPUTS(P0,Q0,Pold,Qold)
      INTEGER I
      REAL P0,Q0,Pold,Qold
      DO 10 I=1,10
      WRITE(9,*)' '
10    CONTINUE
      WRITE(9,*)'     Seidel iteration is used to find the fixed'
      WRITE(9,*)' '
      WRITE(9,*)'     point (P,Q) of the non-linear system'
      WRITE(9,*)' '
      WRITE(9,*)'     P = G (P,Q), Q = G (P,Q). The functions are:'
      WRITE(9,*)'          1            2'
      WRITE(9,*)' '
      CALL PRINTFUN
      WRITE(9,*)'     ENTER the initial starting point (P ,Q )'
      WRITE(9,*)'                                        0  0 '
      WRITE(9,*)' '
      WRITE(9,*)'     P(0) = '
      READ(9,*) Pold
      P0=Pold
      WRITE(9,*)' '
      WRITE(9,*)'     Q(0) = '
      READ(9,*) Qold
      Q0=Qold
      RETURN
      END

      SUBROUTINE RESULTS(P0,Q0,Pold,Qold,Pnew,Qnew,Tol,Sep,Count)
      INTEGER Count,K
      REAL P0,Pnew,Pold,Q0,Qnew,Qold,Sep,Tol
      K=Count
      WRITE(9,*)' '
      WRITE(9,*)' '
      WRITE(9,*)'Seidel iteration was used to find the fixed point of th
     +e'
      WRITE(9,*)' '
      WRITE(9,*)'non-linear system  P = G (P,Q),  Q = G (P,Q)  where:'
      WRITE(9,*)'                        1             2'
      CALL PRINTFUN
      WRITE(9,*)'starting with  P =',P0,'   and   Q =',Q0
      WRITE(9,*)'                0                   0'
      WRITE(9,*)' '
      IF (Sep.LT.Tol) THEN
        WRITE(9,*)'after ',Count,' iterations Seidel`s method was'
        WRITE(9,*)' '
        WRITE(9,*)'successful and found a solution point (P,Q).'
        WRITE(9,*)' '
        WRITE(9,*)'               P =',Pnew,'         Q =',Qnew
        WRITE(9,*)' '
        WRITE(9,*)'              DP =',Pnew-Pold,'        DQ =',Qnew-Qol
     +d
      ELSE
        WRITE(9,*)'Seidel iteration did NOT converge.'
        WRITE(9,*)' '
        WRITE(9,*)'The status after ',Count,' iterations is:'
        WRITE(9,*)' '
        WRITE(9,*)'           P(',K,') =',Pnew,'     Q(',K,') =',Qnew
        WRITE(9,*)' '
        WRITE(9,*)'              DP =',Pnew-Pold,'        DQ =',Qnew-Qol
     +d
      ENDIF
      RETURN
      END
