      PROGRAM LPOLYNOM
C     ----------------------------------------------------------------
C     Alg5'2.for   FORTRAN program for implementing Algorithm 5.2
C     
C     NUMERICAL METHODS: FORTRAN Programs, (c) John H. Mathews 1995
C     To accompany the text:
C     NUMERICAL METHODS for Math., Science & Engineering, 2nd Ed, 1992
C     Prentice Hall, Englewood Cliffs, New Jersey, 07632, U.S.A.
C     Prentice Hall, Inc.; USA, Canada, Mexico ISBN 0-13-624990-6
C     Prentice Hall, International Editions:   ISBN 0-13-625047-5
C     This free software is compliments of the author.
C     E-mail address:       in%"mathews@fullerton.edu"
C     
C     Algorithm 5.2 (Least Squares Polynomial).
C     Section   5.2, Curve Fitting, Page 278
C     ----------------------------------------------------------------
      PARAMETER(MaxN=12,MaxM=100)
      INTEGER M,N
      REAL A,B,C,P,X,Y
      DIMENSION A(1:MaxN,1:MaxN),B(1:MaxN),C(1:MaxN),X(1:MaxM),Y(1:MaxM)
      CHARACTER ANS*1
      EXTERNAL P
10    CALL GETDATA(X,Y,M,N)
      CALL FILLMAT(X,Y,M,A,B,N)
      CALL SOLVELI(A,B,N,C)
20    CALL RESULTS(C,X,Y,M,N)
      CALL EVALUATE(C,N)
      WRITE(9,*)' '
      WRITE(9,*)'WANT TO EVALUATE P(T) AGAIN ? '
      READ(9,'(A)') ANS
      IF(ANS .EQ. 'Y' .OR. ANS .EQ. 'y') GOTO 20
      WRITE(9,*)' '
      WRITE(9,*)'WANT TO FIND ANOTHER LEAST SQUARES POLYNOMIAL ? '
      READ(9,'(A)') ANS
      IF(ANS .EQ. 'Y' .OR. ANS .EQ. 'y') GOTO 10
      STOP
      END

      SUBROUTINE FILLMAT(X,Y,M,A,B,N)
      PARAMETER(MaxN=12,MaxM=100)
      INTEGER Col,J,K,M,N,R
      REAL A,B,Pow,Prod,X,XK,Y,YK
      DIMENSION A(1:MaxN,1:MaxN),B(1:MaxN),X(1:MaxM),Y(1:MaxM)
      DIMENSION Pow(0:MaxN)
      DO R=1,(N+1)
        B(R)=0
      ENDDO
      DO K=1,M
        YK=Y(K)
        XK=X(K)
        Prod=1
        DO R=1,(N+1)
          B(R)=B(R)+YK*Prod
          Prod=Prod*XK
        ENDDO
      ENDDO
      DO J=1,(2*N)
        Pow(J)=0
      ENDDO
      Pow(0)=M
      DO K=1,M
        XK=X(K)
        Prod=X(K)
        DO J=1,(2*N)
          Pow(J)=Pow(J)+Prod
          Prod=Prod*XK
        ENDDO
      ENDDO
      DO R=1,(N+1)
        DO Col=1,(N+1)
          A(R,Col)=Pow(R+Col-2)
        ENDDO
      ENDDO
      RETURN
      END

      SUBROUTINE SOLVELI(A,B,N,C)
      PARAMETER(MaxN=12)
      INTEGER Col,J,K,N,P,Row,T
      REAL A,B,C,Sum
      DIMENSION A(1:MaxN,1:MaxN),B(1:MaxN),C(1:MaxN)
      DIMENSION Row(1:MaxN),Z(1:MaxN)
      DO J=1,(N+1)
        Row(J)=J
      ENDDO
      DO P=1,N
        DO K=(P+1),(N+1)
          IF (ABS(A(Row(K),P)).GT.ABS(A(Row(P),P))) THEN
            T=Row(P)
            Row(P)=Row(K)
            Row(K)=T
          ENDIF
        ENDDO
        DO K=(P+1),(N+1)
          A(Row(K),P)=A(Row(K),P)/A(Row(P),P)
          DO Col=(P+1),(N+1)
            A(Row(K),Col)=A(Row(K),Col)-A(Row(K),P)*A(Row(P),Col)
          ENDDO
        ENDDO
      ENDDO
      Z(1)=B(Row(1))
      DO K=2,(N+1)
        Sum=0
        DO Col=1,(K-1)
          Sum=Sum+A(Row(K),Col)*Z(Col)
        ENDDO
        Z(K)=B(Row(K))-Sum
      ENDDO
      C(N+1)=Z(N+1)/A(Row(N+1),N+1)
      DO K=N,1,-1
        Sum=0
        DO Col=(K+1),(N+1)
          Sum=Sum+A(Row(K),Col)*C(Col)
        ENDDO
        C(K)=(Z(K)-Sum)/A(Row(K),K)
      ENDDO
      RETURN
      END

      REAL FUNCTION P(C,N,T)
      PARAMETER(MaxN=12)
      INTEGER K,N
      REAL C,Sum,T
      DIMENSION C(1:MaxN)
        Sum=C(N+1)
        DO K=N,1,-1
          Sum=C(K)+Sum*T
        ENDDO
        P=Sum
        RETURN
      END


      SUBROUTINE XILLMAT(X,Y,M,A,B,N)
      PARAMETER(MaxN=12,MaxM=100)
      INTEGER Col,J,K,M,N,R
      REAL A,B,Pow,Prod,X,XK,Y,YK
      DIMENSION A(1:MaxN,1:MaxN),B(1:MaxN),X(1:MaxM),Y(1:MaxM)
      DIMENSION Pow(0:MaxN)
      DO 10  R=1,(N+1)
        B(R)=0
10    CONTINUE
      DO 30 K=1,M
        YK=Y(K)
        XK=X(K)
        Prod=1
        DO 20 R=1,(N+1)
          B(R)=B(R)+YK*Prod
          Prod=Prod*XK
20      CONTINUE
30    CONTINUE
      DO 40 J=1,(2*N)
        Pow(J)=0
40    CONTINUE
      Pow(0)=M
      DO 60 K=1,M
        XK=X(K)
        Prod=X(K)
        DO 50 J=1,(2*N)
          Pow(J)=Pow(J)+Prod
          Prod=Prod*XK
50      CONTINUE
60    CONTINUE
      DO 80 R=1,(N+1)
        DO 70 Col=1,(N+1)
          A(R,Col)=Pow(R+Col-2)
70      CONTINUE
80    CONTINUE
      RETURN
      END

      SUBROUTINE XOLVELI(A,B,N,C)
      PARAMETER(MaxN=12)
      INTEGER Col,J,K,N,P,Row,T
      REAL A,B,C,Sum
      DIMENSION A(1:MaxN,1:MaxN),B(1:MaxN),C(1:MaxN)
      DIMENSION Row(1:MaxN),Z(1:MaxN)
      DO 10 J=1,(N+1)
        Row(J)=J
10    CONTINUE
      DO 50 P=1,N
        DO 20 K=(P+1),(N+1)
          IF (ABS(A(Row(K),P)).GT.ABS(A(Row(P),P))) THEN
            T=Row(P)
            Row(P)=Row(K)
            Row(K)=T
          ENDIF
20      CONTINUE
        DO 40 K=(P+1),(N+1)
          A(Row(K),P)=A(Row(K),P)/A(Row(P),P)
          DO 30 Col=(P+1),(N+1)
            A(Row(K),Col)=A(Row(K),Col)-A(Row(K),P)*A(Row(P),Col)
30        CONTINUE
40      CONTINUE
50    CONTINUE
      Z(1)=B(Row(1))
      DO 70 K=2,(N+1)
        Sum=0
        DO 60 Col=1,(K-1)
          Sum=Sum+A(Row(K),Col)*Z(Col)
60      CONTINUE
        Z(K)=B(Row(K))-Sum
70    CONTINUE
      C(N+1)=Z(N+1)/A(Row(N+1),N+1)
      DO 90 K=N,1,-1
        Sum=0
        DO 80 Col=(K+1),(N+1)
          Sum=Sum+A(Row(K),Col)*C(Col)
80      CONTINUE
        C(K)=(Z(K)-Sum)/A(Row(K),K)
90    CONTINUE
      RETURN
      END

      REAL FUNCTION XP(C,N,T)
      PARAMETER(MaxN=12)
      INTEGER K,N
      REAL C,Sum,T
      DIMENSION C(1:MaxN)
      Sum=C(N+1)
      DO 10 K=N,1,-1
        Sum=C(K)+Sum*T
10    CONTINUE
      P=Sum
      RETURN
      END

      SUBROUTINE PRINTPOL(C,N)
      PARAMETER(MaxN=12)
      INTEGER I,K,N
      REAL C
      DIMENSION C(1:MaxN)
      IF (N.EQ.1) THEN
        WRITE(9,*)' '
        WRITE(9,*)'P(X)  =  C X  +  2'
        WRITE(9,*)'          2       1'
      ELSEIF (N.EQ.2) THEN
        WRITE(9,*)'            2'
        WRITE(9,*)'P(X)  =  C X   +  C X  +  C'
        WRITE(9,*)'          3        2       1'
      ELSEIF (N.EQ.3) THEN
        WRITE(9,*)'            3        2'
        WRITE(9,*)'P(X)  =  C X   +  C X   +  C X  +  C'
        WRITE(9,*)'          4        3        2       1'
      ELSE
        WRITE(9,*)'  ',N,'            2'
        WRITE(9,*)'P(X)  =  A C   +...+  C X  +   C X  +  C'
        WRITE(9,*) N,'            3        2       1'
      ENDIF
       WRITE(9,*)' '
      DO 10 K=(N+1),1,-2
        IF (K.GE.2) THEN
          WRITE(9,*)'C(',K,')  =',C(K),'     C(',(K-1),')  =',C(K-1)
        ELSE
          WRITE(9,*)'C(',K,')  =',C(K)
        ENDIF
10    CONTINUE
      RETURN
      END

      SUBROUTINE GETDATA(X,Y,M,N)
      PARAMETER(MaxM=100)
      INTEGER I,K,M,N
      REAL X,Y
      DIMENSION X(1:MaxM),Y(1:MaxM)
      DO 10 I=1,18
      WRITE(9,*)' '
10    CONTINUE
      WRITE(9,*)'THE LEAST SQUARES POLYNOMIAL OF DEGREE N IS CONSTRUCTED
     +'
      WRITE(9,*)' '
      WRITE(9,*)'WHICH FITS THE  M  POINTS (X ,Y ),(X ,Y ),...,(X ,Y ).'
      WRITE(9,*)'                            1  1    2  2        M  M'
      WRITE(9,*)' '
      WRITE(9,*)'ENTER DEGREE  N  OF THE LEAST SQUARES POLYNOMIAL.'
      WRITE(9,*)' '
      WRITE(9,*)'N = '
      READ(9,*) N
      WRITE(9,*)' '
      WRITE(9,*)'ENTER THE NUMBER OF POINTS  M = '
      READ(9,*) M
      IF (M.LT.(N+1)) THEN
        WRITE(9,*)' '
        WRITE(9,*)'YOU MUST HAVE AT LEAST ',N+1,' POINTS.'
        M=N+1
      ENDIF
      WRITE(9,*)' '
      WRITE(9,*)'NOW ENTER THE ',M,' POINTS'
      WRITE(9,*)' '
      WRITE(9,*)'(X ,Y ),(X ,Y ),...,(X ,Y )'
      WRITE(9,*)'  1  1    2  2        M  M'
      WRITE(9,*)' '
      DO 20 K=1,M
        WRITE(9,*)'X(',K,') = '
        READ(9,*) X(K)
        WRITE(9,*)'Y(',K,') = '
        READ(9,*) Y(K)
        WRITE(9,*)' '
20    CONTINUE
      RETURN
      END

      SUBROUTINE RESULTS(C,X,Y,M,N)
      PARAMETER(MaxN=12)
      INTEGER I,K,M,N
      REAL C,Err,P,X,Y,Z
      DIMENSION C(1:MaxN),X(1:MaxM),Y(1:MaxM)
      EXTERNAL P
      DO 10 I=1,18
        WRITE(9,*)' '
10    CONTINUE
      WRITE(9,*)'THE LEAST SQUARES POLYNOMIAL IS:'
      WRITE(9,*)' '
      CALL PRINTPOL(C,N)
      WRITE(9,*)' '
      WRITE(9,*)'    K       X           Y           P(X )        Error'
      WRITE(9,*)'             K           K             K'
      WRITE(9,*)'  -----------------------------------------------------
     +----'
      DO 20 K=1,M
        Z=P(C,N,X(K))
        Err=Y(K)-Z
        WRITE(9,999) K,X(K),Y(K),Z,Err
20    CONTINUE
999   FORMAT(I5,2F12.5,2F14.7)
      RETURN
      END

      SUBROUTINE EVALUATE(C,N)
      PARAMETER(MaxN=12)
      INTEGER N
      REAL C,P,T
      DIMENSION C(1:MaxN)
      EXTERNAL P
      WRITE(9,*)' '
      WRITE(9,*)'NOW EVALUATE P(T) ENTER A VALUE   T = '
      READ(9,*) T
      WRITE(9,*)'P(',T,'  )  = ',P(C,N,T)
      RETURN
      END
