      PROGRAM TRAPEZOD
C     ----------------------------------------------------------------
C     Alg7'1.for   FORTRAN program for implementing Algorithm 7.1
C     
C     NUMERICAL METHODS: FORTRAN Programs, (c) John H. Mathews 1995
C     To accompany the text:
C     NUMERICAL METHODS for Math., Science & Engineering, 2nd Ed, 1992
C     Prentice Hall, Englewood Cliffs, New Jersey, 07632, U.S.A.
C     Prentice Hall, Inc.; USA, Canada, Mexico ISBN 0-13-624990-6
C     Prentice Hall, International Editions:   ISBN 0-13-625047-5
C     This free software is compliments of the author.
C     E-mail address:       in%"mathews@fullerton.edu"
C     
C     Algorithm 7.1 (Composite Trapezoidal Rule).
C     Section   7.2, Composite Trapezoidal & Simpson's Rule, Page 365
C     ----------------------------------------------------------------
      INTEGER M
      REAL A,B,Trule
      CHARACTER*60 ANS*1,DFUN,FUN
      EXTERNAL F
10    CALL INPUTS(A,B,M,DFUN,FUN)
      CALL TRAPRU(F,A,B,M,Trule)
      CALL RESULT(A,B,M,Trule,DFUN,FUN)
      WRITE(9,*)'WANT TO TRY ANOTHER INTERVAL ? <Y/N> '
      READ (9,'(A)') ANS
      IF (ANS.EQ.'Y' .OR. ANS.EQ.'y') GOTO 10
      END

      REAL FUNCTION F(X)
      REAL X
      F=X/(1+X*X)
      RETURN
      END

      SUBROUTINE PRINTFUN(DFUN,FUN)
      CHARACTER*(*) DFUN,FUN
      FUN ='X/(1+X*X)'
      DFUN='X/(1+X*X) DX'
      RETURN
      END

      SUBROUTINE TRAPRU(F,A,B,M,Trule)
      INTEGER K,M
      REAL A,B,H,Sum,Trule,X
      EXTERNAL F
      H=(B-A)/M
      Sum=0
      DO K=1,M-1
        X=A+H*K
        Sum=Sum+F(X)
      ENDDO
      Sum=H*(F(A)+F(B)+2*Sum)/2
      Trule=Sum
      RETURN
      END

      SUBROUTINE XTRAPRU(F,A,B,M,Trule)
      INTEGER K,M
      REAL A,B,H,Sum,Trule,X
      EXTERNAL F
      H=(B-A)/M
      Sum=0
      DO 10 K=1,M-1
        X=A+H*K
        Sum=Sum+F(X)
10    CONTINUE
      Sum=H*(F(A)+F(B)+2*Sum)/2
      Trule=Sum
      RETURN
      END

      SUBROUTINE INPUTS(A,B,M,DFUN,FUN)
      INTEGER I,M
      REAL A,B
      CHARACTER*(*) DFUN,FUN
      CALL PRINTFUN(DFUN,FUN)
      DO 10 I=1,18
        WRITE(9,*)' '
10    CONTINUE
      WRITE(9,*)'    THE TRAPEZOIDAL RULE IS USED TO COMPUTE AN APPROXIM
     +ATION'
      WRITE(9,*)' '
      WRITE(9,*)'FOR THE VALUE OF THE DEFINITE INTEGRAL:'
      WRITE(9,*)' '
      WRITE(9,*)' '
      WRITE(9,*)'    B'
      WRITE(9,*)'    /'
      WRITE(9,*)'    | ',DFUN
      WRITE(9,*)'    /'
      WRITE(9,*)'    A'
      WRITE(9,*)' '
      WRITE(9,*)' '
      WRITE(9,*)'ENTER THE LEFT  ENDPOINT A = '
      READ(9,*) A
      WRITE(9,*)' '
      WRITE(9,*)'ENTER THE RIGHT ENDPOINT B = '
      READ(9,*) B
      WRITE(9,*)' '
      WRITE(9,*)'  NUMBER OF SUBINTERVALS M = '
      READ(9,*) M
      WRITE(9,*)' '
      RETURN
      END

      SUBROUTINE RESULT(A,B,M,Trule,DFUN,FUN)
      INTEGER I,M
      REAL A,B,Trule
      CHARACTER*(*) DFUN,FUN
      CALL PRINTFUN(DFUN,FUN)
      DO 10 I=1,18
        WRITE(9,*)' '
10    CONTINUE
      WRITE(9,*)'            ',B
      WRITE(9,*)'               /'
      WRITE(9,*)'               |'
      WRITE(9,*)Trule,'  ~    |  ',DFUN
      WRITE(9,*)'               |'
      WRITE(9,*)'               /'
      WRITE(9,*)'           ',A
      WRITE(9,*)' '
      WRITE(9,*)' '
      WRITE(9,*)'AN APPROXIMATE VALUE FOR THE DEFINITE INTEGRAL OF'
      WRITE(9,*)' '
      WRITE(9,*)' '
      WRITE(9,*)'F(X) = ',FUN
      WRITE(9,*)' '
      WRITE(9,*)' '
      WRITE(9,*)'TAKEN OVER  [',A,'  ,',B,'  ]  WAS FOUND.'
      WRITE(9,*)' '
      WRITE(9,*)'WHEN ',M,' SUBINTERVALS ARE USED,'
      WRITE(9,*)' '
      WRITE(9,*)'THE TRAPEZOIDAL RULE APPROXIMATION IS ',Trule
      WRITE(9,*)' '
      RETURN
      END
