      PROGRAM GAUSSINT
C     ----------------------------------------------------------------
C     Alg7'6.for   FORTRAN program for implementing Algorithm 7.6
C     
C     NUMERICAL METHODS: FORTRAN Programs, (c) John H. Mathews 1995
C     To accompany the text:
C     NUMERICAL METHODS for Math., Science & Engineering, 2nd Ed, 1992
C     Prentice Hall, Englewood Cliffs, New Jersey, 07632, U.S.A.
C     Prentice Hall, Inc.; USA, Canada, Mexico ISBN 0-13-624990-6
C     Prentice Hall, International Editions:   ISBN 0-13-625047-5
C     This free software is compliments of the author.
C     E-mail address:       in%"mathews@fullerton.edu"
C     
C     Algorithm 7.6 (Gauss-Legendre Quadrature).
C     Section   7.5, Gauss-Legendre Integration, Page 397
C     ----------------------------------------------------------------
      PARAMETER(Tol=1E-6)
      INTEGER I,J
      REAL A,A0,B0,Close,Q,W
      CHARACTER*56 ANS*1,DFUN,FUN
      DIMENSION A(1:17,1:96),Q(1:17),W(1:17,1:96)
      EXTERNAL F
      DO 10 I=1,18
        WRITE(9,*)' '
10    CONTINUE
      WRITE(9,*)'Now loading abscissas and weights. Please wait!'
      CALL READAW(A,W)
20    CALL INPUTS(A0,B0,DFUN,FUN)
      CALL GAUSQUAD(F,A0,B0,A,W,Q,Close,J)
      CALL RESULT(A0,B0,Close,Q,J,FUN,DFUN)
      WRITE(9,*)'Wand to try another interval ? <Y/N> '
      READ (9,'(A)') ANS
      IF (ANS.EQ.'Y'.OR.ANS.EQ.'y') GOTO 20
      END

      REAL FUNCTION F(X)
      REAL X
      F=SIN(X)
      RETURN
      END

      SUBROUTINE PRINTFUN(DFUN,FUN)
      CHARACTER*(*) DFUN,FUN
      FUN ='SIN(X)'
      DFUN='SIN(X) DX'
      RETURN
      END

      SUBROUTINE GAUSQUAD(F,A0,B0,A,W,Q,Close,J)
      PARAMETER(Min=3)
      INTEGER I,J,K
      REAL A,A0,B0,Close,Mid,Sum,Q,W,Wide,X
      DIMENSION A(1:17,1:96),Q(1:17),W(1:17,1:96)
      EXTERNAL F
      Mid=(A0+B0)/2
      Wide=(B0-A0)/2
      Close=1
      J=1
      X= Mid+A(1,1)
      Q(1)= W(1,1)*F(X)*Wide
      WHILE ((Close.GT.Tol.OR.J.LT.Min).AND.J.LT.17)
        J=J+1
        Sum=0
        I=J
        IF (J.GT.10) I=12+4*(J-11)
        IF (J.GT.14) I=24+8*(J-14)
        DO K=1,I
            X=Mid+A(J,K)*Wide
            Sum=Sum+W(J,K)*F(X)
        ENDDO
        Q(J)=Sum*Wide
        Close=ABS(Q(J)-Q(J-1))
      REPEAT
      RETURN
      END

      SUBROUTINE XAUSQUAD(F,A0,B0,A,W,Q,Close,J)
      PARAMETER(Min=3)
      INTEGER I,J,K
      REAL A,A0,B0,Close,Mid,Sum,Q,W,Wide,X
      DIMENSION A(1:17,1:96),Q(1:17),W(1:17,1:96)
      EXTERNAL F
      Mid=(A0+B0)/2
      Wide=(B0-A0)/2
      Close=1
      J=1
      X= Mid+A(1,1)
      Q(1)= W(1,1)*F(X)*Wide
10    IF ((Close.GT.Tol.OR.J.LT.Min).AND.J.LT.17) THEN
        J=J+1
        Sum=0
        I=J
        IF (J.GT.10) I=12+4*(J-11)
        IF (J.GT.14) I=24+8*(J-14)
        DO 20 K=1,I
            X=Mid+A(J,K)*Wide
            Sum=Sum+W(J,K)*F(X)
20      CONTINUE
        Q(J)=Sum*Wide
        Close=ABS(Q(J)-Q(J-1))
        GOTO 10
      ENDIF
      RETURN
      END

      SUBROUTINE READAW(A,W)
      INTEGER I,J,K
      REAL A,M,W
      CHARACTER*56 ANS*1,DFUN,FUN
      DIMENSION A(1:17,1:96),Q(1:17),W(1:17,1:96)
      OPEN(4,FILE='ALG7-6AW.DAT',STATUS='OLD')
      DO 50 J=1,17
        M=J
        I=J
        IF (J.GT.10) THEN
          M=12+4*(J-11)
          I=INT(M)
        ENDIF
        IF (J.GT.14) THEN
          M=24+8*(J-14)
          I=INT(M)
        ENDIF
      DO 10 K=1,INT((M+1)/2)
        READ(4,*) A(J,K)
10    CONTINUE
      DO 20 K=1,INT(M/2)
        A(J,I+1-K)=-A(J,K)
20    CONTINUE
      DO 30 K=1,INT((M+1)/2)
        READ(4,*) W(J,K)
30    CONTINUE
      DO 40 K=1,INT(M/2)
          W(J,I+1-K)=W(J,K)
40    CONTINUE
50    CONTINUE
      CLOSE(4,STATUS='KEEP')
      RETURN
      END

      SUBROUTINE INPUTS(A0,B0,DFUN,FUN)
      INTEGER I
      REAL A0,B0
      CHARACTER*(*) DFUN,FUN
      CALL PRINTFUN(DFUN,FUN)
      DO 10 I=1,18
        WRITE(9,*)' '
10    CONTINUE
      WRITE(9,*)' '
      WRITE(9,*)'          Gaussian quadrature is performed to find an'
      WRITE(9,*)' '
      WRITE(9,*)'     approximation for the value of the definite integr
     +al:'
      WRITE(9,*)' '
      WRITE(9,*)'           B              N          '
      WRITE(9,*)'           /                         '
      WRITE(9,*)'           | F(X) DX  ~  SUM w F(x ) '
      WRITE(9,*)'           /                  k   k  '
      WRITE(9,*)'           A             k=1         '
      WRITE(9,*)' '
      WRITE(9,*)'    You chose to approximate the definite integral:'
      WRITE(9,*)' '
      WRITE(9,*)' '
      WRITE(9,*)'           B'
      WRITE(9,*)'           /'
      WRITE(9,*)'           | ',DFUN
      WRITE(9,*)'           /'
      WRITE(9,*)'           A'
      WRITE(9,*)' '
      WRITE(9,*)' '
      WRITE(9,*)'ENTER the left  endpoint A = '
      READ(9,*) A0
      WRITE(9,*)' '
      WRITE(9,*)'ENTER the right endpoint B = '
      READ(9,*) B0
      WRITE(9,*)' '
      RETURN
      END

      SUBROUTINE RESULT(A0,B0,Close,Q,J,FUN,DFUN)
      INTEGER I,II,J,JJ,K,L,U,V
      REAL A0,B0,Close,Q
      CHARACTER*(*) DFUN,FUN
      DIMENSION Q(1:17)
      CALL PRINTFUN(DFUN,FUN)
      DO 10 I=1,18
        WRITE(9,*)' '
10    CONTINUE
      JJ=J
      IF (J.GT.10) THEN
        JJ=12+4*(J-11)
      ENDIF
      IF (J.GT.14) THEN
        JJ=24+8*(J-14)
      ENDIF
      WRITE(9,*)'           ',B0
      WRITE(9,*)'                   /'
      WRITE(9,*)'                   |'
      WRITE(9,*)Q(J),' ~         |  ',DFUN
      WRITE(9,*)'                   |'
      WRITE(9,*)'                   /'
      WRITE(9,*)'           ',A0
      WRITE(9,*)' '
      WRITE(9,*)' '
      WRITE(9,*)'Gaussian quadrature with',JJ,' points was used to inte
     +grate'
      WRITE(9,*)' '
      WRITE(9,*)'F(X) = ',FUN
      WRITE(9,*)' '
      WRITE(9,*)' '
      WRITE(9,*)'over the interval   [',A0,'  ,',B0,'  ].'
      WRITE(9,*)' '
      WRITE(9,*)'The values of the previous Gaussian rules are:'
      L=1
      IF (J.GT.14) THEN
        L=2
      ENDIF
      IF (J.GT.16) THEN
        L=3
      ENDIF
      DO 20 K=L,INT((J+1)/2)
      U=2*K-1
      V=2*K
      I=U
      IF (U.GT.10) THEN
        I=12+4*(U-11)
      ENDIF
      IF (U.GT.14) THEN
        I=24+8*(U-14)
      ENDIF
      II=V
      IF (V.GT.10) THEN
        II=12+4*(V-11)
      ENDIF
      IF (V.GT.14) THEN
        II=24+8*(V-14)
      ENDIF
      IF (U.LE.J.AND.V.LE.J) THEN
        WRITE(9,998) I,Q(U),II,Q(V)
      ENDIF
      IF (U.LE.J.AND.V.GT.J) THEN
        WRITE(9,999) I,Q(U)
      ENDIF
20    CONTINUE
      WRITE(9,*)' '
      WRITE(9,*)'The best Gaussian approximation is Q(',JJ,') =',Q(J)
      IF (J.LE.11) THEN
        WRITE(9,*)' '
      ENDIF
      WRITE(9,*)'                 The error estimate is  +-',Close
998   FORMAT(' Q(',I2,') = ',F15.7,
     +'        Q(',I2,') = ',F15.7)
999   FORMAT(' Q(',I2,') = ',F15.7)
      RETURN
      END
