      PROGRAM NELDMEAD
C     ----------------------------------------------------------------
C     Alg8'2.for   FORTRAN program for implementing Algorithm 8.2
C     
C     NUMERICAL METHODS: FORTRAN Programs, (c) John H. Mathews 1995
C     To accompany the text:
C     NUMERICAL METHODS for Math., Science & Engineering, 2nd Ed, 1992
C     Prentice Hall, Englewood Cliffs, New Jersey, 07632, U.S.A.
C     Prentice Hall, Inc.; USA, Canada, Mexico ISBN 0-13-624990-6
C     Prentice Hall, International Editions:   ISBN 0-13-625047-5
C     This free software is compliments of the author.
C     E-mail address:       in%"mathews@fullerton.edu"
C     
C     Algorithm 8.2 (Nelder-Mead's Minimization Method).
C     Section   8.1, Minimization of a Function, Page 414
C     ----------------------------------------------------------------
      PARAMETER(MaxN=6,Epsilon=1E-5)
      INTEGER Count,Hi,J,K,Lo,N
      REAL Norm,V,Y
      CHARACTER ANS*1
      DIMENSION V(0:MaxN,1:MaxN),Y(1:MaxN)
      EXTERNAL F
10    CALL INPUT(F,V,Y,N)
      CALL NELDER(F,V,Y,N,Epsilon,Norm,Lo,Hi,Count)
      CALL OUTPUT(V,Y,N,Norm,Lo,Hi,Count)
      WRITE(9,*)' '
      WRITE(9,*)'WANT TO TRY NEW STARTING VERTICES ? '
      READ(9,'(A)') ANS
      IF (ANS.EQ.'Y' .OR. ANS.EQ.'y') GOTO 10
      END

      REAL FUNCTION F(P)
      PARAMETER(MaxN=6)
      REAL P,X,Y,Z,U,V,W
      DIMENSION P(1:MaxN)
        X=P(1)
        Y=P(2)
        Z=P(3)
        U=P(4)
        V=P(5)
        W=P(6)
        F=X*X - 4*X + Y*Y - Y - X*Y
      RETURN
      END

      SUBROUTINE PRINTFUN(N)
      INTEGER K,N
      CHARACTER C*2,FUN*80,L*6,R*4
      DIMENSION C(1:6)
        N = 2
      FUN ='X*X - 4*X + Y*Y - Y - X*Y'
      C(1)='(X'
      C(2)=',Y'
      C(3)=',Z'
      C(4)=',U'
      C(5)=',V'
      C(6)=',W,'
        L ='     F'
        R =') = '
      WRITE(9,999) L,C(1),(C(K),K=2,N),R,FUN
999   FORMAT(9A)
      RETURN
      END

      SUBROUTINE NELDER(F,V,Y,N,Epsilon,Norm,Lo,Hi,Count)
      PARAMETER(MaxN=6,Min=10,Max=200)
      INTEGER Count,Hi,Ho,J,K,Li,Lo,N
      REAL C,E,Epsilon,M,Norm,R,S,V,Y,YM,YC,YE,YR,Z
      DIMENSION C(1:MaxN),E(1:MaxN),M(1:MaxN),R(1:MaxN)
      DIMENSION V(0:MaxN,1:MaxN),Y(1:MaxN),Z(1:MaxN)
      EXTERNAL F
      Count=0
      Lo=0                       !Order the vertices:
      Hi=0
      DO J=1,N
        IF (Y(J).LT.Y(Lo)) Lo=J
        IF (Y(J).GT.Y(Hi)) Hi=J
      ENDDO
      Li=Hi
      Ho=Lo
      DO J=0,N
        IF (J.NE.Lo .AND. Y(J).LT.Y(Li)) Li=J
        IF (J.NE.Hi .AND. Y(J).GT.Y(Ho)) Ho=J
      ENDDO
200   WHILE (Y(Hi).GT.(Y(Lo)+Epsilon) .AND.
     +      (Count.LT.Max .OR. Count.LT.Min))
        DO K=1,N                 !The main loop starts here:
          S=0                    !Form the new points:
          DO J=0,N
            S=S+V(J,K)
          ENDDO
          M(K)=(S-V(Hi,K))/N
        ENDDO
        DO K=1,N
          R(K)=2*M(K)-V(Hi,K)
        ENDDO
        YR=F(R)
        IF (YR.LT.Y(Ho)) THEN    !Improve the simplex:
          IF (Y(Li).LT.YR) THEN
            DO K=1,N             !Replace a vertex:
              V(Hi,K)=R(K)
            ENDDO
            Y(Hi)=YR
          ELSE
            DO K=1,N
              E(K)=2*R(K)-M(K)
            ENDDO
            YE=F(E)
              IF (YE.LT.Y(Li)) THEN
                DO K=1,N
                  V(Hi,K)=E(K)
                ENDDO
                Y(Hi)=YE
              ELSE
                DO K=1,N         !Replace a vertex:
                  V(Hi,K)=R(K)
                ENDDO
                Y(Hi)=YR
              ENDIF
          ENDIF
        ELSE
          IF (YR.LT.Y(Hi)) THEN
            DO K=1,N             !Replace a vertex:
              V(Hi,K)=R(K)
            ENDDO
            Y(Hi)=YR
          ENDIF
          DO K=1,N
            C(K)=(V(Hi,K)+M(K))/2
          ENDDO
          YC=F(C)
          IF (YC.LT.Y(Hi)) THEN
            DO K=1,N
              V(Hi,K)=C(K)
            ENDDO
            Y(Hi)=YC
          ELSE
            DO J=0,N             !Shrink the simplex:
              IF (J.NE.Lo) THEN
                DO K=1,N
                  V(J,K)=(V(J,K)+V(Lo,K))/2
                  Z(K)=V(J,K)
                ENDDO
                Y(J)=F(Z)
              ENDIF
            ENDDO
          ENDIF
        ENDIF
        Count=Count+1
        Lo=0                     !Order the vertices:
        Hi=0
        DO J=1,N
          IF (Y(J).LT.Y(Lo)) Lo=J
          IF (Y(J).GT.Y(Hi)) Hi=J
        ENDDO
        Li=Hi
        Ho=Lo
        DO J=0,N
          IF (J.NE.Lo .AND. Y(J).LT.Y(Li)) Li=J
          IF (J.NE.Hi .AND. Y(J).GT.Y(Ho)) Ho=J
        ENDDO
      REPEAT                     !End of the main loop.
      Norm=0                     !Determine the size of the simplex:
      DO J=0,N
        S=0
        DO K=1,N
          S=S+(V(Lo,K)-V(J,K))*(V(Lo,K)-V(J,K))
        ENDDO
        IF (S.GT.Norm) Norm=S
      ENDDO
      Norm=SQRT(Norm)
      RETURN
      END

      SUBROUTINE XNELDER(F,V,Y,N,Epsilon,Norm,Lo,Hi,Count)
      PARAMETER(MaxN=6,Min=10,Max=200)
      INTEGER Count,Hi,Ho,J,K,Li,Lo,N
      REAL C,E,Epsilon,M,Norm,R,S,V,Y,YM,YC,YE,YR,Z
      DIMENSION C(1:MaxN),E(1:MaxN),M(1:MaxN),R(1:MaxN)
      DIMENSION V(0:MaxN,1:MaxN),Y(1:MaxN),Z(1:MaxN)
      EXTERNAL F
      Count=0
      Lo=0                       !Order the vertices:
      Hi=0
      DO 100 J=1,N
        IF (Y(J).LT.Y(Lo)) Lo=J
        IF (Y(J).GT.Y(Hi)) Hi=J
100   CONTINUE
      Li=Hi
      Ho=Lo
      DO 110 J=0,N
        IF (J.NE.Lo .AND. Y(J).LT.Y(Li)) Li=J
        IF (J.NE.Hi .AND. Y(J).GT.Y(Ho)) Ho=J
110   CONTINUE
200   IF (Y(Hi).GT.(Y(Lo)+Epsilon) .AND.
     +   (Count.LT.Max .OR. Count.LT.Min)) THEN
        DO 310 K=1,N             !The main loop starts here:
          S=0                    !Form the new points:
          DO 300 J=0,N
            S=S+V(J,K)
300       CONTINUE
          M(K)=(S-V(Hi,K))/N
310     CONTINUE
        DO 320 K=1,N
          R(K)=2*M(K)-V(Hi,K)
320     CONTINUE
        YR=F(R)
        IF (YR.LT.Y(Ho)) THEN    !Improve the simplex:
          IF (Y(Li).LT.YR) THEN
            DO 400 K=1,N         !Replace a vertex:
              V(Hi,K)=R(K)
400         CONTINUE
            Y(Hi)=YR
          ELSE
            DO 410 K=1,N
              E(K)=2*R(K)-M(K)
410         CONTINUE
            YE=F(E)
              IF (YE.LT.Y(Li)) THEN
                DO 420 K=1,N
                  V(Hi,K)=E(K)
420             CONTINUE
                Y(Hi)=YE
              ELSE
                DO 430 K=1,N     !Replace a vertex:
                  V(Hi,K)=R(K)
430             CONTINUE
                Y(Hi)=YR
              ENDIF
          ENDIF
        ELSE
          IF (YR.LT.Y(Hi)) THEN
            DO 440 K=1,N         !Replace a vertex:
              V(Hi,K)=R(K)
440         CONTINUE
            Y(Hi)=YR
          ENDIF
          DO 450 K=1,N
            C(K)=(V(Hi,K)+M(K))/2
450       CONTINUE
          YC=F(C)
          IF (YC.LT.Y(Hi)) THEN
            DO 460 K=1,N
              V(Hi,K)=C(K)
460         CONTINUE
            Y(Hi)=YC
          ELSE
            DO 480 J=0,N         !Shrink the simplex:
              IF (J.NE.Lo) THEN
                DO 470 K=1,N
                  V(J,K)=(V(J,K)+V(Lo,K))/2
                  Z(K)=V(J,K)
470             CONTINUE
                Y(J)=F(Z)
              ENDIF
480         CONTINUE
          ENDIF
        ENDIF
        Count=Count+1
        Lo=0                     !Order the vertices:
        Hi=0
        DO 500 J=1,N
          IF (Y(J).LT.Y(Lo)) Lo=J
          IF (Y(J).GT.Y(Hi)) Hi=J
500     CONTINUE
        Li=Hi
        Ho=Lo
        DO 510 J=0,N
          IF (J.NE.Lo .AND. Y(J).LT.Y(Li)) Li=J
          IF (J.NE.Hi .AND. Y(J).GT.Y(Ho)) Ho=J
510     CONTINUE
        GOTO 200
      ENDIF                      !End of the main loop.
      Norm=0                     !Determine the size of the simplex:
      DO 610 J=0,N
        S=0
        DO 600 K=1,N
          S=S+(V(Lo,K)-V(J,K))*(V(Lo,K)-V(J,K))
600     CONTINUE
        IF (S.GT.Norm) Norm=S
610   CONTINUE
      Norm=SQRT(Norm)
      RETURN
      END

      SUBROUTINE INPUT(F,V,Y,N)
      PARAMETER(MaxN=6)
      INTEGER I,J,K,N
      REAL V,Y,Z
      DIMENSION V(0:MaxN,1:MaxN),Y(1:MaxN),Z(1:MaxN)
      EXTERNAL F
      DO 10 I=1,16
        WRITE(9,*)' '
10    CONTINUE
      WRITE(9,*)'   THE NELDER-MEAD SIMPLEX METHOD OR "POLYTOPE METHOD"
     + IS'
      WRITE(9,*)' '
      WRITE(9,*)'USED FOR FINDING THE MINIMUM OF THE FUNCTION  F(V)'
      WRITE(9,*)' '
      WRITE(9,*)'FOR CONVENIENCE, THE FUNCTION F(V) CAN BE EXPRESSED USI
     +NG'
      WRITE(9,*)' '
      WRITE(9,*)'THE VARIABLES X = v , Y = v , Z = v , U = v , V = v , W
     + = v .'
      WRITE(9,*)'                   1       2       3       4       5
     +    6 '
      WRITE(9,*)' '
      CALL PRINTFUN(N)
      WRITE(9,*)' '
      WRITE(9,*)'YOU MUST SUPPLY',N+1,' LINEARLY INDEPENDENT'
      WRITE(9,*)' '
      IF (N.EQ.2) THEN
        WRITE(9,*)'STARTING POINTS   V = (v   ,v   )   FOR  J=0,1,3'
        WRITE(9,*)'                   J    J,1  J,2'
      ELSEIF (N.EQ.3) THEN
        WRITE(9,*)'STARTING POINTS   V = (v   ,v   ,v   )   FOR  J=0,1,3
     +,4'
        WRITE(9,*)'                   J    J,1  J,2  J,3'
      ELSE
        WRITE(9,*)'STARTING POINTS   V = (v   ,v   ,...,v   )   FOR  J=0
     +,1,...,N'
        WRITE(9,*)'                   J    J,1  J,2      J,N'
        WRITE(9,*)'WHERE N =',N
      ENDIF
      DO 30 J=0,N
        WRITE(9,*)' '
        WRITE(9,*)'GIVE COORDINATES OF POINT  V'
        WRITE(9,*)'                  ',J
      DO 20 K=1,N
        WRITE(9,*)'V(',J,',',K,') = '
        READ(9,*) V(J,K)
        Z(K)=V(J,K)
20    CONTINUE
      Y(J)=F(Z)
30    CONTINUE
      RETURN
      END

      SUBROUTINE OUTPUT(V,Y,N,Norm,Lo,Hi,Count)
      PARAMETER(MaxN=6,Epsilon=1E-5)
      INTEGER Count,Hi,I,K,Lo,N
      REAL Norm,V,Y
      DIMENSION V(0:MaxN,1:MaxN),Y(1:MaxN)
      DO 10 I=1,16
        WRITE(9,*)' '
10    CONTINUE
      WRITE(9,*)' '
      WRITE(9,*)'   THE NELDER-MEAD METHOD WAS USED TO FIND THE MINIMUM
     + OF THE FUNCTION'
      WRITE(9,*)' '
      CALL PRINTFUN(N)
      WRITE(9,*)' '
      WRITE(9,*)'IT TOOK ',Count,' ITERATIONS TO FIND AN APPROXIMATION F
     +OR'
      WRITE(9,*)' '
      IF (N.EQ.2) THEN
        WRITE(9,*)'THE COORDINATES OF THE LOCAL MINIMUM  P = (p ,p )'
        WRITE(9,*)'                                            1  2'
      ELSEIF (N.EQ.3) THEN
        WRITE(9,*)'THE COORDINATES OF THE LOCAL MINIMUM  P = (p ,p ,p )'
        WRITE(9,*)'                                            1  2  3'
      ELSE
        WRITE(9,*)'THE COORDINATES OF THE LOCAL MINIMUM  P = (p ,p ,...,
     +p )'
        WRITE(9,*)'                                            1  2
     + N'
        WRITE(9,*)'WHERE N =',N
      ENDIF
      DO 20 K=1,N
        WRITE(9,*)'P(',K,')  =',V(Lo,K)
20    CONTINUE
      WRITE(9,*)' '
      WRITE(9,*)'THE MAXIMUM DISTANCE TO THE OTHER VERTICES OF THE SIMP
     +LEX IS'
      WRITE(9,*)' '
      WRITE(9,*)'            DP  =',Norm
      WRITE(9,*)' '
      WRITE(9,*)'THE FUNCTION VALUE AT THE MINIMUM POINT IS'
      WRITE(9,*)' '
      WRITE(9,*)'F(P)  =',Y(Lo)
      WRITE(9,*)' '
      WRITE(9,*)'  DF  =',Y(Hi)-Y(Lo),'  IS AN ESTIMATE FOR THE ACCURACY
     +.'
      RETURN
      END
