      PROGRAM QMINUM
C     ----------------------------------------------------------------
C     Alg8'3.for   FORTRAN program for implementing Algorithm 8.3
C     
C     NUMERICAL METHODS: FORTRAN Programs, (c) John H. Mathews 1995
C     To accompany the text:
C     NUMERICAL METHODS for Math., Science & Engineering, 2nd Ed, 1992
C     Prentice Hall, Englewood Cliffs, New Jersey, 07632, U.S.A.
C     Prentice Hall, Inc.; USA, Canada, Mexico ISBN 0-13-624990-6
C     Prentice Hall, International Editions:   ISBN 0-13-625047-5
C     This free software is compliments of the author.
C     E-mail address:       in%"mathews@fullerton.edu"
C     
C     Algorithm 8.3 (Local Minimum Search: Quadratic Interpolation).
C     Section   8.1, Minimization of a Function, Page 416
C     ----------------------------------------------------------------
      PARAMETER(Delta=1E-5,Epsilon=1E-7,Jmax=50,Kmax= 50)
      INTEGER Cond,Count
      REAL Error,H,P0,Pmin,Pold,Y0,Ymin
      CHARACTER ANS*1
      EXTERNAL F,F1
10    CALL INPUTS(P0,Pold)
      CALL QMINIM(F,F1,P0,Delta,Epsilon,Jmax,Kmax,Pmin,H,Ymin,
     +Error,Cond,Count)
      CALL RESULT(Pold,Pmin,H,Ymin,Error,Epsilon,Cond,Count)
      WRITE(9,*)' '
      WRITE(9,*)'WANT TO TRY ANOTHER STARTING POINT ? '
      READ(9,'(A)') ANS
      IF (ANS.EQ.'Y' .OR. ANS.EQ.'y') GOTO 10
      END

      REAL FUNCTION F(X)
      REAL X
      F=X*X*X-X
      RETURN
      END

      REAL FUNCTION F1(X)
      REAL X
      F1=3*X*X-1
      RETURN
      END

      SUBROUTINE PRINTFUN
      WRITE(9,*)'F(X)  =  X*X*X-X'
      RETURN
      END

      SUBROUTINE QMINIM(F,F1,P0,Delta,Epsilon,Jmax,Kmax,Pmin,H,Ymin,
     +Error,Cond,Count)
      PARAMETER(Big=1E9)
      INTEGER Cond,Count,J,Jmax,K,Kmax
      REAL Delta,Epsilon,Error,H,P0,Pmin,Ymin
      REAL D,E0,E1,E2,H0,H1,H2,Hmin,P1,P2,Y0,Y1,Y2
      EXTERNAL F,F1
      K=0
      Error=1
      Cond=0
      Count=0
      H=1
      IF (ABS(P0).GT.1E4) H=ABS(P0)/1E4
      WHILE (K.LT.Kmax .AND. Error.GT.Epsilon .AND. Cond.NE.5)
        IF (F1(P0).GT.0) H=-ABS(H)
        P1=P0+H
        P2=P0+2*H
        Pmin= P0
        Y0=F(P0)
        Y1=F(P1)
        Y2=F(P2)
        Ymin=Y0
        Cond=0
        J=0
        WHILE (J.LT.Jmax .AND. ABS(H).GT.Delta .AND. Cond.EQ.0)
          IF (Y0.LE.Y1) THEN
            P2=P1
            Y2=Y1
            H=H/2
            P1=P0+H
            Y1=F(P1)
          ELSE
            IF (Y2.LT.Y1) THEN
              P1=P2
              Y1=Y2
              H=2*H
              P2=P0+2*H
              Y2=F(P2)
            ELSE
              Cond=-1
            ENDIF
          ENDIF
          J=J+1
          Count=Count+1
          IF (ABS(H).GT.Big .OR. ABS(P0).GT.Big) Cond=5
          WRITE(9,1000) Count,P1,Y1
        REPEAT
        IF (Cond.EQ.5) THEN
          Pmin=P1
          Ymin=F(P1)
        ELSE
          D=4*Y1-2*Y0-2*Y2                      !Start of a long block:
          IF (D.LT.0) THEN
            Hmin=H*(4*Y1-3*Y0-Y2)/D
          ELSE
            Hmin=H/3
            Cond=4
          ENDIF
          Pmin=P0+Hmin
          Ymin=F(Pmin)
          H=ABS(H)
          H0=ABS(Hmin)
          H1=ABS(Hmin-H)
          H2=ABS(Hmin-2*H)
          IF (H0.LT.H) H=H0
          IF (H1.LT.H) H=H1
          IF (H2.LT.H) H=H2
          IF (H.EQ.0)  H=Hmin
          IF (H.LT.Delta) Cond=1
          IF (ABS(H).GT.Big .OR. ABS(Pmin).GT.Big) Cond=5
          E0=ABS(Y0-Ymin)
          E1=ABS(Y1-Ymin)
          E2=ABS(Y2-Ymin)
          IF (E0.NE.0 .AND. E0.LT.Error) Error=E0
          IF (E1.NE.0 .AND. E1.LT.Error) Error=E1
          IF (E2.NE.0 .AND. E2.LT.Error) Error=E2
          IF (E0.EQ.0 .AND. E1.EQ.0 .AND. E2.EQ.0) Error=0
          IF (Error.LT.Epsilon) Cond=2
          P0=Pmin
          K=K+1
          Count=Count+1
        ENDIF                                   !End of the long block.
        WRITE(9,1000) Count,Pmin,Ymin
        IF (Cond.EQ.2 .AND. H.LT.Delta) Cond=3
      REPEAT
      RETURN
1000  FORMAT(5X,I4,5X,F18.7,5X,F18.7)
      END

      SUBROUTINE XMINIM(F,F1,P0,Delta,Epsilon,Jmax,Kmax,Pmin,H,Ymin,
     +Error,Cond,Count)
      PARAMETER(Big=1E9)
      INTEGER Cond,Count,J,Jmax,K,Kmax
      REAL Delta,Epsilon,Error,H,P0,Pmin,Ymin
      REAL D,E0,E1,E2,H0,H1,H2,Hmin,P1,P2,Y0,Y1,Y2
      EXTERNAL F,F1
      K=0
      Error=1
      Cond=0
      Count=0
      H=1
      IF (ABS(P0).GT.1E4) H=ABS(P0)/1E4
10    IF (K.LT.Kmax .AND. Error.GT.Epsilon .AND. Cond.NE.5) THEN
        IF (F1(P0).GT.0) H=-ABS(H)
        P1=P0+H
        P2=P0+2*H
        Pmin= P0
        Y0=F(P0)
        Y1=F(P1)
        Y2=F(P2)
        Ymin=Y0
        Cond=0
        J=0
20      IF (J.LT.Jmax .AND. ABS(H).GT.Delta .AND. Cond.EQ.0) THEN
          IF (Y0.LE.Y1) THEN
            P2=P1
            Y2=Y1
            H=H/2
            P1=P0+H
            Y1=F(P1)
          ELSE
            IF (Y2.LT.Y1) THEN
              P1=P2
              Y1=Y2
              H=2*H
              P2=P0+2*H
              Y2=F(P2)
            ELSE
              Cond=-1
            ENDIF
          ENDIF
          J=J+1
          Count=Count+1
          IF (ABS(H).GT.Big .OR. ABS(P0).GT.Big) Cond=5
          WRITE(9,1000) Count,P1,Y1
          GOTO 20
        ENDIF
        IF (Cond.EQ.5) THEN
          Pmin=P1
          Ymin=F(P1)
        ELSE
          D=4*Y1-2*Y0-2*Y2                      !Start of a long block:
          IF (D.LT.0) THEN
            Hmin=H*(4*Y1-3*Y0-Y2)/D
          ELSE
            Hmin=H/3
            Cond=4
          ENDIF
          Pmin=P0+Hmin
          Ymin=F(Pmin)
          H=ABS(H)
          H0=ABS(Hmin)
          H1=ABS(Hmin-H)
          H2=ABS(Hmin-2*H)
          IF (H0.LT.H) H=H0
          IF (H1.LT.H) H=H1
          IF (H2.LT.H) H=H2
          IF (H.EQ.0)  H=Hmin
          IF (H.LT.Delta) Cond=1
          IF (ABS(H).GT.Big .OR. ABS(Pmin).GT.Big) Cond=5
          E0=ABS(Y0-Ymin)
          E1=ABS(Y1-Ymin)
          E2=ABS(Y2-Ymin)
          IF (E0.NE.0 .AND. E0.LT.Error) Error=E0
          IF (E1.NE.0 .AND. E1.LT.Error) Error=E1
          IF (E2.NE.0 .AND. E2.LT.Error) Error=E2
          IF (E0.EQ.0 .AND. E1.EQ.0 .AND. E2.EQ.0) Error=0
          IF (Error.LT.Epsilon) Cond=2
          P0=Pmin
          K=K+1
          Count=Count+1
        ENDIF                                   !End of the long block.
        WRITE(9,1000) Count,Pmin,Ymin
        GOTO 10
      ENDIF
      IF (Cond.EQ.2 .AND. H.LT.Delta) Cond=3
      PAUSE
      RETURN
1000  FORMAT(5X,I4,5X,F18.7,5X,F18.7)
      END

      SUBROUTINE INPUTS(P0,Pold)
      INTEGER I
      REAL P0
      WRITE(9,*)' '
      WRITE(9,*)'     THE QUADRATIC SEARCH METHOD IS USED'
      WRITE(9,*)' '
      WRITE(9,*)'TO FIND A LOCAL MINIMUM OF THE FUNCTION: '
      WRITE(9,*)' '
      CALL PRINTFUN
      WRITE(9,*)' '
      WRITE(9,*)'ONE INITIAL APPROXIMATION P0 IS NEEDED.'
      WRITE(9,*)' '
      WRITE(9,*)'ENTER P0 = '
      READ(9,*) P0
      Pold=P0
      WRITE(9,*)' '
      RETURN
      END

      SUBROUTINE RESULT(Pold,Pmin,H,Ymin,Error,Epsilon,Cond,Count)
      INTEGER Cond,Count,I
      REAL Error,Epsilon,H,Pold,Pmin,Ymin
      DO 10 I=1,18
        WRITE(9,*)' '
10    CONTINUE
      WRITE(9,*)'     THE QUADRATIC SEARCH METHOD WAS USED '
      WRITE(9,*)' '
      WRITE(9,*)'TO FIND A LOCAL MINIMUM OF THE FUNCTION: '
      WRITE(9,*)' '
      CALL PRINTFUN
      WRITE(9,*)' '
      WRITE(9,*)'STARTING WITH THE APPROXIMATION  P0 =',Pold
      WRITE(9,*)' '
      WRITE(9,*)'AFTER ',Count,' ITERATIONS AN APPROXIMATION FOR THE MIN
     +IMUM IS:'
      WRITE(9,*)' '
      WRITE(9,*)'     P  =',Pmin
      WRITE(9,*)' '
      WRITE(9,*)'    DP  =',ABS(H),'  IS THE ESTIMATED ACCURACY FOR P.'
      WRITE(9,*)' '
      WRITE(9,*)'       F(',Pmin,'  )  =',Ymin
      WRITE(9,*)' '
      IF (Cond.EQ.0) THEN
        WRITE(9,*)'CONVERGENCE IS DOUBTFUL BECAUSE THE'
        WRITE(9,*)' '
        WRITE(9,*)'MAXIMUM NUMBER OF ITERATIONS WAS EXCEEDED.'
      ELSEIF (Cond.EQ.1) THEN
        WRITE(9,*)'CONVERGENCE HAS BEEN ACHIEVED BECAUSE '
        WRITE(9,*)' '
        WRITE(9,*)'CONSECUTIVE ABSCISSAS ARE CLOSER THAN ',Delta
      ELSEIF (Cond.EQ.2) THEN
        WRITE(9,*)'CONVERGENCE HAS BEEN ACHIEVED BECAUSE '
        WRITE(9,*)' '
        WRITE(9,*)'CONSECUTIVE ORDINATES ARE CLOSER THAN ',Epsilon
      ELSEIF (Cond.EQ.3) THEN
        WRITE(9,*)'CONVERGENCE HAS BEEN ACHIEVED BECAUSE'
        WRITE(9,*)' '
        WRITE(9,*)'CONSECUTIVE ABSCISSAS ARE CLOSER THAN ',H
        WRITE(9,*)' '
        WRITE(9,*)'CONSECUTIVE ORDINATES ARE CLOSER THAN ',Error
      ELSEIF (Cond.EQ.4) THEN
        WRITE(9,*)'CONVERGENCE IS DOUBTFUL BECAUSE DIVISION BY ZERO WAS
     +ENCOUNTERED.'
        IF (Error.LT.Epsilon/100) THEN
          WRITE(9,*)'HOWEVER, CONSECUTIVE ORDINATES ARE CLOSE.'
        ENDIF
      ELSEIF (Cond.EQ.5) THEN
        WRITE(9,*)'CONVERGENCE IS DOUBTFUL, H IS TOO LARGE, H  =',H
        WRITE(9,*)' '
        WRITE(9,*)'PERHAPS A DIFFERENT STARTING VALUE SHOULD BE USED.'
        WRITE(9,*)' '
        WRITE(9,*)'IT IS POSSIBLE THAT THERE IS NO LOCAL MINIMUM.'
      ENDIF
      RETURN
      END
