      PROGRAM SOLVEDE
C     ----------------------------------------------------------------
C     Alg9'1-4.for   FORTRAN program for implementing Algorithm 9.1-4
C     
C     NUMERICAL METHODS: FORTRAN Programs, (c) John H. Mathews 1995
C     To accompany the text:
C     NUMERICAL METHODS for Math., Science & Engineering, 2nd Ed, 1992
C     Prentice Hall, Englewood Cliffs, New Jersey, 07632, U.S.A.
C     Prentice Hall, Inc.; USA, Canada, Mexico ISBN 0-13-624990-6
C     Prentice Hall, International Editions:   ISBN 0-13-625047-5
C     This free software is compliments of the author.
C     E-mail address:       in%"mathews@fullerton.edu"
C     
C     Algorithm 9.1 (Euler's Method). 
C     Section   9.2, Euler's Method, Page 435
C     
C     Algorithm 9.2 (Heun's Method).
C     Section   9.3, Heun's Method, Page 441
C     
C     Algorithm 9.3 (Taylor's Method of Order 4).
C     Section   9.4, Taylor Series Method, Page 448
C     
C     Algorithm 9.4 (Runge-Kutta Method of Order 4).
C     Section   9.5, Runge-Kutta Methods, Page 460
C     ----------------------------------------------------------------
      PARAMETER(MaxM=1000)
      INTEGER DoMo,M,Mend,Meth,Order,State
      REAL A,B,D,T,TK,Y,YK,Y0
      DIMENSION D(1:4),T(0:1000),Y(0:1000)
      CHARACTER Ans*1
      EXTERNAL DF,F
      Meth=1
      A=0
      B=0
      Y0=0
      M=1
      State=1
10    CONTINUE
      CALL MESSAGE(Meth,Order,State)
      DoMo=1
20      CONTINUE
          CALL INPUT(Meth)
30        CONTINUE
            CALL EPOINTS(A,B,Y0,M,State)
            IF (Meth.EQ.1) THEN
              CALL EULER(F,A,B,Y0,M,Mend,T,Y)
            ELSEIF (Meth.EQ.2) THEN
              CALL CAUCHY(F,A,B,Y0,M,Mend,T,Y)
            ELSEIF (Meth.EQ.3) THEN
              CALL HEUN(F,A,B,Y0,M,Mend,T,Y)
            ELSEIF (Meth.EQ.4) THEN
              CALL TAYLOR(DF,A,B,Y0,M,Mend,T,TK,Y,YK,D,Order)
            ELSEIF (Meth.EQ.5) THEN
              CALL TAYLOR(DF,A,B,Y0,M,Mend,T,TK,Y,YK,D,Order)
            ELSEIF (Meth.EQ.6) THEN
              CALL RUNGE(F,A,B,Y0,M,Mend,T,Y)
            ENDIF
            CALL RESULTS(T,Y,M,Mend,Meth)
            WRITE(9,*)' '
            WRITE(9,*)'      Want to try another initial condition ?
     & <Y/N> '
            READ(9,'(A)') Ans
            IF (Ans.NE.'Y'.AND.Ans.NE.'y') THEN
              State=2
            ELSE
              State=0
            ENDIF
          IF (State.EQ.0.OR.State.EQ.1) GOTO 30
          WRITE(9,*)'   Want to change the differential equation ?
     & <Y/N> '
            READ(9,'(A)') Ans
          IF (Ans.NE.'Y'.AND.Ans.NE.'y') THEN
            DoMo=0
          ELSE
            State=0
          ENDIF
        IF (DoMo.EQ.1) GOTO 20
      WRITE(9,*)'Want to try another method of approximation ? <Y/N> '
      READ(9,'(A)') Ans
      IF (Ans.NE.'Y'.AND.Ans.NE.'y') THEN
        Meth=0
      ELSE
        State=0
      ENDIF
      IF (Meth.NE.0) GOTO 10
      RETURN
      END

      REAL FUNCTION F(T,Y)
      REAL T,Y
      F=T**2+Y**2
      RETURN
      END

      SUBROUTINE DF(T,Y,D)
      REAL D,T,Y
      DIMENSION D(1:4)
        D(1) = ( T - Y )/2
        D(2) = ( 2 - T + Y)/4
        D(3) = (-2 + T - Y)/8
        D(4) = ( 2 - T + Y)/16
        RETURN
      END

      SUBROUTINE PRINTFUN
        WRITE(9,*)'          Y` = T**2+Y**2'
      RETURN
      END

      SUBROUTINE EULER(F,A,B,Y0,M,Mend,T,Y)
      PARAMETER(Big=1E13)
      INTEGER K,M,Mend
      REAL A,B,H,T,Y,Y0
      DIMENSION T(0:1000),Y(0:1000)
      EXTERNAL F
      H=(B-A)/M
      T(0)=A
      Y(0)=Y0
      DO K=0,M-1
        Y(K+1)=Y(K)+H*F(T(K),Y(K))
        T(K+1)=A+H*(K+1)
        Mend=K+1
        IF (Big.LT.ABS(Y(K+1))) RETURN
      ENDDO
      RETURN
      END

      SUBROUTINE HEUN(F,A,B,Y0,M,Mend,T,Y)
      PARAMETER(Big=1E13)
      INTEGER J,M,Mend
      REAL A,B,H,K1,K2,P,T,Y,Y0
      DIMENSION T(0:1000),Y(0:1000)
      EXTERNAL F
      H=(B-A)/M
      T(0)=A
      Y(0)=Y0
      DO J=0,M-1
        K1=F(T(J),Y(J))
        P=Y(J)+H*K1
        T(J+1)=A+H*(J+1)
        Mend=J+1
        IF (Big.LT.ABS(P)) THEN
          Y(J+1)=P
          RETURN
        ENDIF
        K2=F(T(J+1),P)
        Y(J+1)=Y(J)+H*(K1+K2)/2
        IF (Big.LT.ABS(Y(J+1))) RETURN
      ENDDO
      RETURN
      END

      SUBROUTINE CAUCHY(F,A,B,Y0,M,Mend,T,Y)
      PARAMETER(Big=1E13)
      INTEGER J,M,Mend
      REAL A,B,H,K1,K2,P,T,TP,Y,Y0
      DIMENSION T(0:1000),Y(0:1000)
      EXTERNAL F
      H=(B-A)/M
      T(0)=A
      Y(0)=Y0
      DO J=0,M-1
        K1=F(T(J),Y(J))
        P=Y(J)+H*K1/2
        T(J+1)=A+H*(J+1)
        Mend=J+1
        IF (Big.LT.ABS(P)) THEN
          Y(J+1)=P
          RETURN
        ENDIF
        TP=T(J)+H/2
        K2=F(TP,P)
        Y(J+1)=Y(J)+H*K2
        IF (Big.LT.ABS(Y(J+1))) RETURN
      ENDDO
      RETURN
      END

      SUBROUTINE TAYLOR(DF,A,B,Y0,M,Mend,T,TK,Y,YK,D,Order)
      PARAMETER(Big=1E13)
      INTEGER J,M,Mend,Order
      REAL A,B,D,H,T,TK,Y,YK,Y0
      DIMENSION D(1:4),T(0:1000),Y(0:1000)
      EXTERNAL DF
      H=(B-A)/M
      T(0)=A
      Y(0)=Y0
      DO K=0,M-1
        TK=T(K)
        YK=Y(K)
        CALL DF(TK,YK,D)
        IF (Order.EQ.3) THEN
          Y(K+1)=YK+H*(D(1)+H*(D(2)/2+H*D(3)/6))
        ELSE
          Y(K+1)=YK+H*(D(1)+H*(D(2)/2+H*(D(3)/6+H*D(4)/24)))
        ENDIF
        T(K+1)=A+H*(K+1)
        Mend=K+1
        IF (Big.LT.ABS(Y(K+1))) RETURN
      ENDDO
      RETURN
      END

      SUBROUTINE RUNGE(F,A,B,Y0,M,Mend,T,Y)
      PARAMETER(Big=1E13)
      INTEGER J,M,Mend,Pole
      REAL A,B,H,K1,K2,K3,K4,P,T,TJ,Y,YJ,Y0
      DIMENSION T(0:1000),Y(0:1000)
      EXTERNAL F
      H=(B-A)/M
      T(0)=A
      Y(0)=Y0
      DO J=0,M-1
        TJ=T(J)
        YJ=Y(J)
        K1=H*F(TJ,YJ)
          IF (Big.LT.ABS(YJ+0.5*K1)) RETURN
        K2=H*F(TJ+H/2,YJ+0.5*K1)
          IF (Big.LT.ABS(YJ+0.5*K2)) RETURN
        K3=H*F(TJ+H/2,YJ+0.5*K2)
          IF (Big.LT.ABS(YJ+K3)) RETURN
        K4=H*F(TJ+H,YJ+K3)
        Y(J+1)=YJ+(K1+2*K2+2*K3+K4)/6
        T(J+1)=A+H*(J+1)
        Mend=J+1
          IF (Big.LT.ABS(Y(J+1))) RETURN
      ENDDO
      RETURN
      END

      SUBROUTINE XUNGE(F,A,B,Y0,M,Mend,T,Y)
      PARAMETER(Big=1E13)
      INTEGER J,M,Mend,Pole
      REAL A,B,H,K1,K2,K3,K4,P,T,TJ,Y,YJ,Y0
      DIMENSION T(0:1000),Y(0:1000)
      EXTERNAL F
      Pole=0
      H=(B-A)/M
      T(0)=A
      Y(0)=Y0
      DO J=0,M-1
        TJ=T(J)
        YJ=Y(J)
        Mend=J+1
        K1=H*F(TJ,YJ)
        IF (Big.LT.ABS(YJ+0.5*K1)) THEN
          Pole=1
          GOTO 10
        ENDIF
        K2=H*F(TJ+H/2,YJ+0.5*K1)
        IF (Big.LT.ABS(YJ+0.5*K2)) THEN
          Pole=1
          GOTO 10
        ENDIF
        K3=H*F(TJ+H/2,YJ+0.5*K2)
        IF (Big.LT.ABS(YJ+K3)) THEN
          Pole=1
          GOTO 10
        ENDIF
        K4=H*F(TJ+H,YJ+K3)
        Y(J+1)=YJ+(K1+2*K2+2*K3+K4)/6
        T(J+1)=A+H*(J+1)
        IF (Big.LT.ABS(Y(J+1))) GOTO 10
      ENDDO
10    CONTINUE
      IF (Pole.EQ.0) RETURN
      P=YJ+K1
      T(J+1)=A+H*(J+1)
      IF (Big.LT.ABS(P)) THEN
        Y(J+1)=P
        RETURN
      ENDIF
      K2=F(T(J+1),P)
      Y(J+1)=YJ+H*(K1+K2)/2
      RETURN
      END

      SUBROUTINE XEULER(F,A,B,Y0,M,Mend,T,Y)
      PARAMETER(Big=1E13)
      INTEGER K,M,Mend
      REAL A,B,H,T,Y,Y0
      DIMENSION T(0:1000),Y(0:1000)
      EXTERNAL F
      H=(B-A)/M
      T(0)=A
      Y(0)=Y0
      DO 10 K=0,M-1
        Y(K+1)=Y(K)+H*F(T(K),Y(K))
        T(K+1)=A+H*(K+1)
        Mend=K+1
        IF (Big.LT.ABS(Y(K+1))) GOTO 20
10    CONTINUE
20    CONTINUE
      RETURN
      END

      SUBROUTINE XHEUN(F,A,B,Y0,M,Mend,T,Y)
      PARAMETER(Big=1E13)
      INTEGER J,M,Mend
      REAL A,B,H,K1,K2,P,T,Y,Y0
      DIMENSION T(0:1000),Y(0:1000)
      EXTERNAL F
      H=(B-A)/M
      T(0)=A
      Y(0)=Y0
      DO 10 J=0,M-1
      K1=F(T(J),Y(J))
      P=Y(J)+H*K1
      T(J+1)=A+H*(J+1)
      Mend=J+1
      IF (Big.LT.ABS(P)) THEN
        Y(J+1)=P
        GOTO 20
      ENDIF
      K2=F(T(J+1),P)
      Y(J+1)=Y(J)+H*(K1+K2)/2
      IF (Big.LT.ABS(Y(J+1))) GOTO 20
10    CONTINUE
20    CONTINUE
      RETURN
      END

      SUBROUTINE XCAUCHY(F,A,B,Y0,M,Mend,T,Y)
      PARAMETER(Big=1E13)
      INTEGER J,M,Mend
      REAL A,B,H,K1,K2,P,T,TP,Y,Y0
      DIMENSION T(0:1000),Y(0:1000)
      EXTERNAL F
      H=(B-A)/M
      T(0)=A
      Y(0)=Y0
      DO 10 J=0,M-1
      K1=F(T(J),Y(J))
      P=Y(J)+H*K1/2
      T(J+1)=A+H*(J+1)
      Mend=J+1
      IF (Big.LT.ABS(P)) THEN
        Y(J+1)=P
        GOTO 20
      ENDIF
      TP=T(J)+H/2
      K2=F(TP,P)
      Y(J+1)=Y(J)+H*K2
      IF (Big.LT.ABS(Y(J+1))) GOTO 20
10    CONTINUE
20    CONTINUE
      RETURN
      END

      SUBROUTINE XTAYLOR(DF,A,B,Y0,M,Mend,T,TK,Y,YK,D,Order)
      PARAMETER(Big=1E13)
      INTEGER J,M,Mend,Order
      REAL A,B,D,H,T,TK,Y,YK,Y0
      DIMENSION D(1:4),T(0:1000),Y(0:1000)
      EXTERNAL DF
      H=(B-A)/M
      T(0)=A
      Y(0)=Y0
      DO 10 K=0,M-1
      TK=T(K)
      YK=Y(K)
      CALL DF(TK,YK,D)
      IF (Order.EQ.3) THEN
        Y(K+1)=YK+H*(D(1)+H*(D(2)/2+H*D(3)/6))
      ELSE
        Y(K+1)=YK+H*(D(1)+H*(D(2)/2+H*(D(3)/6+H*D(4)/24)))
      ENDIF
      T(K+1)=A+H*(K+1)
      Mend=K+1
      IF (Big.LT.ABS(Y(K+1))) GOTO 20
10    CONTINUE
20    CONTINUE
      RETURN
      END

      SUBROUTINE XRUNGE(F,A,B,Y0,M,Mend,T,Y)
      PARAMETER(Big=1E13)
      INTEGER J,M,Mend,Pole
      REAL A,B,H,K1,K2,K3,K4,P,T,TJ,Y,YJ,Y0
      DIMENSION T(0:1000),Y(0:1000)
      EXTERNAL F
      Pole=0
      H=(B-A)/M
      T(0)=A
      Y(0)=Y0
      DO 10 J=0,M-1
        TJ=T(J)
        YJ=Y(J)
        Mend=J+1
        K1=H*F(TJ,YJ)
        IF (Big.LT.ABS(YJ+0.5*K1)) THEN
          Pole=1
          GOTO 20
        ENDIF
        K2=H*F(TJ+H/2,YJ+0.5*K1)
        IF (Big.LT.ABS(YJ+0.5*K2)) THEN
          Pole=1
          GOTO 20
        ENDIF
        K3=H*F(TJ+H/2,YJ+0.5*K2)
        IF (Big.LT.ABS(YJ+K3)) THEN
          Pole=1
          GOTO 20
        ENDIF
        K4=H*F(TJ+H,YJ+K3)
        Y(J+1)=YJ+(K1+2*K2+2*K3+K4)/6
        T(J+1)=A+H*(J+1)
        IF (Big.LT.ABS(Y(J+1))) GOTO 20
10    CONTINUE
20    CONTINUE
      IF (Pole.EQ.0) GOTO 30
      P=YJ+K1
      T(J+1)=A+H*(J+1)
      IF (Big.LT.ABS(P)) THEN
        Y(J+1)=P
        GOTO 30
      ENDIF
      K2=F(T(J+1),P)
      Y(J+1)=YJ+H*(K1+K2)/2
30    CONTINUE
      RETURN
      END

      SUBROUTINE MESSAGE(Meth,Order,State)
      INTEGER K,Meth,Order,State
      REAL Resp
      DO 10 I=1,15
        WRITE(9,*)' '
10    CONTINUE
      WRITE(9,*)'          Solution of the differential equation  Y` = F
     +(T,Y)'
      WRITE(9,*)' '
      WRITE(9,*)'          with the initial condition  Y(A) = Y .'
      WRITE(9,*)'                                              0'
      WRITE(9,*)' '
      WRITE(9,*)'          A numerical approximation is computed over [A
     +,B].'
      WRITE(9,*)' '
      WRITE(9,*)' '
      WRITE(9,*)'          Choose the method of approximation:'
      WRITE(9,*)' '
      WRITE(9,*)'    < 1 > Euler`s method'
      WRITE(9,*)' '
      WRITE(9,*)'    < 2 > Modified Euler-Cauchy method'
      WRITE(9,*)' '
      WRITE(9,*)'    < 3 > Heun`s method'
      WRITE(9,*)' '
      WRITE(9,*)'    < 4 > Taylor`s method of order N=3'
      WRITE(9,*)' '
      WRITE(9,*)'    < 5 > Taylor`s method of order N=4'
      WRITE(9,*)' '
      WRITE(9,*)'    < 6 > Runge-Kutta method of order N=4'
      WRITE(9,*)' '
      WRITE(9,*)'          SELECT < 1 - 6 > ? '
      Resp=Meth
      READ(9,*) Resp
      Meth=INT(Resp)
      IF ((Meth.LT.1.OR.Meth.GT.6).AND.State.NE.0) THEN
        Meth=1
      ENDIF
      IF (Meth.EQ.4) THEN
        Order=3
      ENDIF
      IF (Meth.EQ.5) THEN
        Order=4
      ENDIF
      RETURN
      END

      SUBROUTINE INPUT(Meth)
      INTEGER Meth
      REAL Valu
      CHARACTER Ans*1
      DO 10 I=1,15
        WRITE(9,*)' '
10    CONTINUE
      WRITE(9,*)'     You chose '
      WRITE(9,*)' '
      IF (Meth.EQ.1) THEN
        WRITE(9,*)'     Euler`s method to solve the D.E.'
      ELSEIF (Meth.EQ.2) THEN
        WRITE(9,*)'     the Modified Euler-Cauchy method to solve
     + the D.E.'
      ELSEIF (Meth.EQ.3) THEN
        WRITE(9,*)'     Heun`s method to solve the D.E.'
      ELSEIF (Meth.EQ.4) THEN
        WRITE(9,*)'     Taylor`s method of order N=3 to solve
     + the D.E.'
      ELSEIF (Meth.EQ.5) THEN
        WRITE(9,*)'     Taylor`s method of order N=4 to solve
     + the D.E.'
      ELSEIF (Meth.EQ.6) THEN
        WRITE(9,*)'     the Runge-Kutta method of order N=4 to solve
     + the D.E.'
      ENDIF
      WRITE(9,*)' '
      CALL PRINTFUN
      WRITE(9,*)' '
      WRITE(9,*)'     with the initial condition Y(A) = Y0.'
      WRITE(9,*)' '
      WRITE(9,*)'     A numerical approximation is computed over [A,B].'
      WRITE(9,*)' '
      WRITE(9,*)'     You must enter the endpoints for the interval,'
      WRITE(9,*)' '
      WRITE(9,*)'     the initial condition Y0, and the number of steps
     + M.'
      WRITE(9,*)' '
      WRITE(9,*)' '
      WRITE(9,*)'         Press the <ENTER> key. '
      READ(9,'(A)') Ans
      RETURN
      END

      SUBROUTINE EPOINTS(A,B,Y0,M,State)
      INTEGER I,M,Stat,State
      REAL A,B,Y0,Valu
C     STATUS = (Change,Enter,Done)
      CHARACTER Ans*1
      Stat=1
      IF (State.EQ.0) THEN
        Stat=0
      ENDIF
10    CONTINUE
      DO 20 I=1,15
        WRITE(9,*)' '
20    CONTINUE
      CALL PRINTFUN
      WRITE(9,*)' '
      WRITE(9,*)' '
      IF (Stat.EQ.1) THEN
        WRITE(9,*)'     ENTER  the  left  endpoint  A = '
        READ(9,*) A
        WRITE(9,*)' '
        WRITE(9,*)'     ENTER  the  right endpoint  B = '
        READ(9,*) B
        WRITE(9,*)' '
        WRITE(9,*)'     ENTER initial condition  Y(A) = '
        READ(9,*) Y0
        WRITE(9,*)' '
        WRITE(9,*)'     ENTER the number of steps   M = '
        Valu=M
        READ(9,*) Valu
        M=INT(Valu)
        IF (M.LT.1) THEN
          M=1
        ENDIF
        IF (M.GT.1000) THEN
          M=1000
        ENDIF
      ELSE
        WRITE(9,*)'     The left  endpoint is    A =',A
        WRITE(9,*)' '
        WRITE(9,*)'     The right endpoint is    B =',B
        WRITE(9,*)' '
        WRITE(9,*)'     Initial condition is  Y(A) =',Y0
        WRITE(9,*)' '
        WRITE(9,*)'     The number of steps is   M = ',M
      ENDIF
      WRITE(9,*)' '
      WRITE(9,*)' '
      WRITE(9,*)'     Do you want to make a change ?  <Y/N>  '
      READ(9,'(A)') Ans
      IF (Ans.EQ.'Y'.OR.Ans.EQ.'y') THEN
          Stat=0
          DO 30 I=1,15
            WRITE(9,*)' '
30        CONTINUE
          WRITE(9,*)'          '
          CALL PRINTFUN
          WRITE(9,*)' '
          WRITE(9,*)' '
          WRITE(9,*)'     The current left  endpoint is A =',A
            WRITE(9,*)'     ENTER  the NEW left  endpoint A = '
          READ(9,*) A
          WRITE(9,*)' '
          WRITE(9,*)'     The current right endpoint is B =',B
            WRITE(9,*)'     ENTER  the NEW right endpoint B = '
          READ(9,*) B
          WRITE(9,*)' '
          WRITE(9,*)'        The current   I. C. is  Y(A) =',Y0
            WRITE(9,*)'        ENTER the NEW    I. C.  Y(A) = '
          READ(9,*) Y0
          WRITE(9,*)' '
          WRITE(9,*)'        The current value of M is  M = ',M
            WRITE(9,*)'        ENTER  the  NEW  value of  M = '
          Valu=M
          READ(9,*) Valu
          M=INT(Valu)
          IF (M.LT.1) THEN
            M=1
          ENDIF
          IF (M.GT.1000) THEN
            M=1000
          ENDIF
      ELSE
        Stat=2
      ENDIF
      IF (Stat.EQ.0.OR.Stat.EQ.1) GOTO 10
      RETURN
      END

      SUBROUTINE RESULTS(T,Y,M,Mend,Meth)
      INTEGER I,K,M,Mend,Meth
      REAL T,Y
      DIMENSION T(0:1000),Y(0:1000)
      DO 10 I=1,10
        WRITE(9,*)' '
10    CONTINUE
      IF (Meth.EQ.1) THEN
        WRITE(9,*)'Euler`s method was used to solve the D.E.'
      ELSEIF (Meth.EQ.2) THEN
        WRITE(9,*)'The Modified Euler-Cauchy method was used to solve th
     +e D.E.'
      ELSEIF (Meth.EQ.3) THEN
        WRITE(9,*)'Heun`s method was used to solve the D.E.'
      ELSEIF (Meth.EQ.4) THEN
        WRITE(9,*)'Taylor`s method of order N=3 was used to solve the D.
     +E.'
      ELSEIF (Meth.EQ.5) THEN
        WRITE(9,*)'Taylor`s method of order N=4 was used to solve the D.
     +E.'
      ELSEIF (Meth.EQ.6) THEN
        WRITE(9,*)'The Runge-Kutta method of order N=4 was used to solve
     + the D.E.'
      ENDIF
      WRITE(9,*)' '
      CALL PRINTFUN
      WRITE(9,*)' '
      WRITE(9,*)'with  Y(',T(0),'  ) =',Y(0)
      WRITE(9,*)' '
      WRITE(9,*)'   K            T(K)           Y(K)'
      WRITE(9,*)' ---------------------------------------'
      DO 20 K=0,Mend
        WRITE(9,999) K, T(K), Y(K)
20    CONTINUE
      IF (Mend.LT.M) THEN
        WRITE(9,*)' '
        WRITE(9,*)'The solution points are approaching a pole.'
      ENDIF
999   FORMAT(I4,4X,4F15.7)
      PAUSE
      RETURN
      END
