      PROGRAM SOLVEDE
C     ----------------------------------------------------------------
C     Alg9'5-8.for   FORTRAN program for implementing Algorithm 9.5-8
C     
C     NUMERICAL METHODS: FORTRAN Programs, (c) John H. Mathews 1995
C     To accompany the text:
C     NUMERICAL METHODS for Math., Science & Engineering, 2nd Ed, 1992
C     Prentice Hall, Englewood Cliffs, New Jersey, 07632, U.S.A.
C     Prentice Hall, Inc.; USA, Canada, Mexico ISBN 0-13-624990-6
C     Prentice Hall, International Editions:   ISBN 0-13-625047-5
C     This free software is compliments of the author.
C     E-mail address:       in%"mathews@fullerton.edu"
C     
C     Algorithm 9.5 (Runge-Kutta-Fehlberg Method RKF45).
C     Section   9.5, Runge-Kutta Methods, Page 461
C     
C     Algorithm 9.6 (Adams-Bashforth-Moulton Method). 
C     Section   9.6, Predictor-Corrector Method, Page 471
C     
C     Algorithm 9.7 (Milne-Simpson Method). 
C     Section   9.6, Predictor-Corrector Method, Page 472
C     
C     Algorithm 9.8 (The Hamming Method).
C     Section   9.6, Predictor-Corrector Method, Page 473
C     ----------------------------------------------------------------
      PARAMETER(MaxM=1000)
      INTEGER DoMo,M,Mend,Meth,Order,State
      REAL A,B,D,T,TK,Tol,Y,YK,Y0
      DIMENSION D(1:4),T(1:1000),Y(1:1000)
      CHARACTER Ans*1
      EXTERNAL DF,F
      Meth=1
      A=0
      B=0
      Y0=0
      M=1
      State=1
10    CONTINUE
      CALL MESSAGE(Meth,Order,State)
      DoMo=1
20      CONTINUE
          CALL INPUT(Meth)
30        CONTINUE
            CALL EPOINTS(A,B,Y0,M,Tol,State)
            IF (Meth.EQ.1) THEN
              CALL ABM(F,A,B,Y0,M,T,Y,Mend)
            ELSEIF (Meth.EQ.2) THEN
              CALL MILNE(F,A,B,Y0,M,T,Y,Mend)
            ELSEIF (Meth.EQ.3) THEN
              CALL HAMMING(F,A,B,Y0,M,T,Y,Mend)
            ELSEIF (Meth.EQ.6) THEN
              CALL RKF45(F,A,B,Y0,Tol,T,Y,M,MaxM,Mend)
            ENDIF
            CALL RESULTS(T,Y,M,Mend,Meth)
            WRITE(9,*)' '
            WRITE(9,*)'      Want to try another initial condition ? <Y/
     +N> '
            READ(9,'(A)') Ans
            IF (Ans.NE.'Y'.AND.Ans.NE.'y') THEN
              State=2
            ELSE
              State=0
            ENDIF
          IF (State.EQ.0.OR.State.EQ.1) GOTO 30
          WRITE(9,*)'   Want to change the differential equation ? <Y/N>
     + '
            READ(9,'(A)') Ans
          IF (Ans.NE.'Y'.AND.Ans.NE.'y') THEN
            DoMo=0
          ELSE
            State=0
          ENDIF
        IF (DoMo.EQ.1) GOTO 20
      WRITE(9,*)'Want to try another method of approximation ? <Y/N> '
      READ(9,'(A)') Ans
      IF (Ans.NE.'Y'.AND.Ans.NE.'y') THEN
        Meth=0
      ELSE
        State=0
      ENDIF
      IF (Meth.NE.0) GOTO 10
      RETURN
      END

      REAL FUNCTION F(T,Y)
      REAL T,Y
        F=T**2+Y**2
        RETURN
      END

      SUBROUTINE PRINTFUN
        WRITE(9,*)'          Y` = T**2+Y**2'
        RETURN
      END

      SUBROUTINE HAMMING(F,A,B,Y0,M,T,Y,Mend)
      PARAMETER(Big=1E13)
      REAL A,B,Cnew,Cold,H,F1,F2,F3,F4,Pmod,Pnew,Pold,T,TK,Y,YK,Y0
      REAL K1,K2,K3,K4,TJ,YJ
      DIMENSION T(1:1000),Y(1:1000)
      EXTERNAL F
      H=(B-A)/M
      T(0)=A
      Y(0)=Y0
      !Use Runge-Kutta to compute three starting values.
      DO J=0,2
        TJ=T(J)
        YJ=Y(J)
        K1=H*F(TJ,YJ)
        K2=H*F(TJ+H/2,YJ+0.5*K1)
        K3=H*F(TJ+H/2,YJ+0.5*K2)
        K4=H*F(TJ+H,YJ+K3)
        Y(J+1)=YJ+(K1+2*K2+2*K3+K4)/6
        T(J+1)=A+H*(J+1)
      ENDDO
      !Start of Procedure Hamming
      F1=F(T(1),Y(1))
      F2=F(T(2),Y(2))
      F3=F(T(3),Y(3))
      Pold=0
      Cold=0
      DO K=3,M-1
        Pnew=Y(K-3)+4*H*(2*F1-F2+2*F3)/3
        Pmod=Pnew+112*(Cold-Pold)/121
        T(K+1)=A+H*(K+1)
        F4=F(T(K+1),Pmod)
        Cnew=(9*Y(K)-Y(K-2)+3*H*(-F2+2*F3+F4))/8
        Y(K+1)=Cnew+9*(Pnew-Cnew)/121
        Mend=K+1
        IF (Big.LT.ABS(Y(K+1))) RETURN
        Pold=Pnew
        Cold=Cnew
        F1=F2
        F2=F3
        F3=F(T(K+1),Y(K+1))
      ENDDO
      RETURN
      END

      SUBROUTINE ABM(F,A,B,Y0,M,T,Y,Mend)
      PARAMETER(Big=1E13)
      INTEGER J,K,M
      REAL A,B,H,H2,F0,F1,F2,F3,F4,P,T,TK,Y,YK,Y0
      REAL K1,K2,K3,K4,TJ,YJ
      DIMENSION T(1:1000),Y(1:1000)
      EXTERNAL F
      H=(B-A)/M
      T(0)=A
      Y(0)=Y0
      !Use Runge-Kutta to compute three starting values.
      DO J=0,2
        TJ=T(J)
        YJ=Y(J)
        K1=H*F(TJ,YJ)
        K2=H*F(TJ+H/2,YJ+0.5*K1)
        K3=H*F(TJ+H/2,YJ+0.5*K2)
        K4=H*F(TJ+H,YJ+K3)
        Y(J+1)=YJ+(K1+2*K2+2*K3+K4)/6
        T(J+1)=A+H*(J+1)
      ENDDO
      !Start of Procedure Adams-Bashforth-Moulton
      F0=F(T(0),Y(0))
      F1=F(T(1),Y(1))
      F2=F(T(2),Y(2))
      F3=F(T(3),Y(3))
      H2=H/24
      DO K=3,M-1
        TK=T(K)
        YK=Y(K)
        P=YK+H2*(-9*F0+37*F1-59*F2+55*F3)
        T(K+1)=A+H*(K+1)
        F4=F(T(K+1),P)
        Y(K+1)=YK+H2*(F1-5*F2+19*F3+9*F4)
        Mend=K+1
        IF (Big.LT.ABS(Y(K+1))) RETURN
        F0=F1
        F1=F2
        F2=F3
        F3=F(T(K+1),Y(K+1))
      ENDDO
      RETURN
      END

      SUBROUTINE MILNE(F,A,B,Y0,M,T,Y,Mend)
      PARAMETER(Big=1E13)
      INTEGER J,K,M
      REAL A,B,H,F1,F2,F3,F4,Pmod,Pnew,Pold,T,TK,Y,YK,Yold,Y0
      REAL K1,K2,K3,K4,TJ,YJ
      DIMENSION T(1:1000),Y(1:1000)
      EXTERNAL F
      H=(B-A)/M
      T(0)=A
      Y(0)=Y0
      !Use Runge-Kutta to compute three starting values.
      DO J=0,2
        TJ=T(J)
        YJ=Y(J)
        K1=H*F(TJ,YJ)
        K2=H*F(TJ+H/2,YJ+0.5*K1)
        K3=H*F(TJ+H/2,YJ+0.5*K2)
        K4=H*F(TJ+H,YJ+K3)
        Y(J+1)=YJ+(K1+2*K2+2*K3+K4)/6
        T(J+1)=A+H*(J+1)
      ENDDO
      !Start of Procedure Milne-Simpson
      F1=F(T(1),Y(1))
      F2=F(T(2),Y(2))
      F3=F(T(3),Y(3))
      Pold=0
      Yold=0
      DO K=3,M-1
        Pnew=Y(K-3)+4*H*(2*F1-F2+2*F3)/3
        Pmod=Pnew+28*(Yold-Pold)/29
        T(K+1)=A+H*(K+1)
        F4=F(T(K+1),Pmod)
        Y(K+1)=Y(K-1)+H*(F2+4*F3+F4)/3
        Mend=K+1
        IF (Big.LT.ABS(Y(K+1))) RETURN
        Pold=Pnew
        Yold=Y(K+1)
        F1=F2
        F2=F3
        F3=F(T(K+1),Y(K+1))
      ENDDO
      RETURN
      END

      SUBROUTINE RKF45(F,A,B,Y0,Tol,T,Y,M,MaxM,Mend)
      PARAMETER(Big=1E13)
      INTEGER I,J,K,M,Mend
      REAL A,B,T,Tol,Y,Y0
      REAL Br,H,Err,Hmin,Hmax,K1,K2,K3,K4,K5,K6,TJ,YJ
      REAL A2,B2,A3,B3,C3,A4,B4,C4,D4,A5,B5,C5,D5,E5,S
      REAL A6,B6,C6,D6,E6,F6,R1,R3,R4,R5,R6,N1,N3,N4,N5
      REAL M1,M3,M4,M5,M6,Y1,Y2,Y3,Y4,Y5,Y6,Ygood,Ynew
      DIMENSION T(1:1000),Y(1:1000)
      EXTERNAL F
      DATA A2,B2,A3,B3,C3,A4,B4,C4,D4,A5,B5,C5,D5,E5,
     A     A6,B6,C6,D6,E6,F6,R1,R3,R4,R5,R6,N1,N3,N4,N5
     B/ 0.25,   0.25,   0.375,   0.09375,  0.28125,   0.9230769231,
     C  0.8793809741,  -3.277196177,           3.320892126,    1.0,
     D  2.032407407,   -8.0,   7.173489279,   -0.2058966862,   0.5,
     E -0.2962962963,   2.0,  -1.381676413,    0.4529727096,  -0.275,
     F  0.002777777778,    -0.02994152047,    -0.02919989367,   0.02,
     G  0.03636363636, 0.1157407407, 0.5489278752, 0.535331384, -0.2/
      H=(B-A)/M
      Hmin=H/1024
      Hmax=H*64
      T(0)=A
      Y(0)=Y0
      T(0)=A
      J=0
      TJ=A
      Br=B-0.00001*ABS(B)
      WHILE (T(J).LT.B)
        IF ((T(J)+H).GT.Br) H=B-T(J)
        TJ=T(J)
        YJ=Y(J)
        Y1=YJ
        K1=H*F(TJ,Y1)
        Y2=YJ+B2*K1;                         IF Big.LT.ABS(Y2) RETURN
        K2=H*F(TJ+A2*H,Y2)
        Y3=YJ+B3*K1+C3*K2;                   IF Big.LT.ABS(Y3) RETURN
        K3=H*F(TJ+A3*H,Y3)
        Y4=YJ+B4*K1+C4*K2+D4*K3;             IF Big.LT.ABS(Y4) RETURN
        K4=H*F(TJ+A4*H,Y4)
        Y5=YJ+B5*K1+C5*K2+D5*K3+E5*K4;       IF Big.LT.ABS(Y5) RETURN
        K5=H*F(TJ+A5*H,Y5)
        Y6=YJ+B6*K1+C6*K2+D6*K3+E6*K4+F6*K5; IF Big.LT.ABS(Y6) RETURN
        K6=H*F(TJ+A6*H,Y6)
        Err=ABS(R1*K1+R3*K3+R4*K4+R5*K5+R6*K6)
        Ynew=YJ+N1*K1+N3*K3+N4*K4+N5*K5
        Err=ABS(Err)
        IF (Err.LT.Tol.OR.H.LE.2*Hmin) THEN
          Y(J+1)=Ynew
          IF ((TJ+H).GT.Br) THEN
              T(J+1)=B
          ELSE
              T(J+1)=TJ+H
          ENDIF
          J=J+1
          TJ=T(J)
        ENDIF
        IF (Err.EQ.0) THEN
          S=0
        ELSE
          S=0.84*(Tol*H/Err)**0.25
        ENDIF
        IF (S.LT.0.75.AND.H.GT.(2*Hmin)) H=H/2
        IF (S.GT.1.50.AND.(2*H).LT.Hmax) H=H*2
        IF (Big.LT.ABS(Y(J)).OR.MaxM.EQ.J) RETURN
        Mend=J
        IF (B.GT.T(J)) THEN
          M=J+1
        ELSE
          M=J
        ENDIF
        WRITE(9,1000) T(J),Y(J)
      REPEAT
      RETURN
1000  FORMAT(5X,2F18.9)
      END

      SUBROUTINE XAMMING(F,A,B,Y0,M,T,Y,Mend)
      PARAMETER(Big=1E13)
      REAL A,B,Cnew,Cold,H,F1,F2,F3,F4,Pmod,Pnew,Pold,T,TK,Y,YK,Y0
      REAL K1,K2,K3,K4,TJ,YJ
      DIMENSION T(1:1000),Y(1:1000)
      EXTERNAL F
      H=(B-A)/M
      T(0)=A
      Y(0)=Y0
C     Use Runge-Kutta to compute three starting values.
      DO 10 J=0,2
        TJ=T(J)
        YJ=Y(J)
        K1=H*F(TJ,YJ)
        K2=H*F(TJ+H/2,YJ+0.5*K1)
        K3=H*F(TJ+H/2,YJ+0.5*K2)
        K4=H*F(TJ+H,YJ+K3)
        Y(J+1)=YJ+(K1+2*K2+2*K3+K4)/6
        T(J+1)=A+H*(J+1)
10    CONTINUE
C     Start of Procedure Hamming
      F1=F(T(1),Y(1))
      F2=F(T(2),Y(2))
      F3=F(T(3),Y(3))
      Pold=0
      Cold=0
      DO 20 K=3,M-1
        Pnew=Y(K-3)+4*H*(2*F1-F2+2*F3)/3
        Pmod=Pnew+112*(Cold-Pold)/121
        T(K+1)=A+H*(K+1)
        F4=F(T(K+1),Pmod)
        Cnew=(9*Y(K)-Y(K-2)+3*H*(-F2+2*F3+F4))/8
        Y(K+1)=Cnew+9*(Pnew-Cnew)/121
        Mend=K+1
        IF (Big.LT.ABS(Y(K+1))) RETURN
        Pold=Pnew
        Cold=Cnew
        F1=F2
        F2=F3
        F3=F(T(K+1),Y(K+1))
20    CONTINUE
      RETURN
      END

      SUBROUTINE XABM(F,A,B,Y0,M,T,Y,Mend)
      PARAMETER(Big=1E13)
      INTEGER J,K,M
      REAL A,B,H,H2,F0,F1,F2,F3,F4,P,T,TK,Y,YK,Y0
      REAL K1,K2,K3,K4,TJ,YJ
      DIMENSION T(1:1000),Y(1:1000)
      EXTERNAL F
      H=(B-A)/M
      T(0)=A
      Y(0)=Y0
C     Use Runge-Kutta to compute three starting values.
      DO 10 J=0,2
        TJ=T(J)
        YJ=Y(J)
        K1=H*F(TJ,YJ)
        K2=H*F(TJ+H/2,YJ+0.5*K1)
        K3=H*F(TJ+H/2,YJ+0.5*K2)
        K4=H*F(TJ+H,YJ+K3)
        Y(J+1)=YJ+(K1+2*K2+2*K3+K4)/6
        T(J+1)=A+H*(J+1)
10    CONTINUE
C     Start of Procedure Adams-Bashforth-Moulton
      F0=F(T(0),Y(0))
      F1=F(T(1),Y(1))
      F2=F(T(2),Y(2))
      F3=F(T(3),Y(3))
      H2=H/24
      DO 20 K=3,M-1
        TK=T(K)
        YK=Y(K)
        P=YK+H2*(-9*F0+37*F1-59*F2+55*F3)
        T(K+1)=A+H*(K+1)
        F4=F(T(K+1),P)
        Y(K+1)=YK+H2*(F1-5*F2+19*F3+9*F4)
        Mend=K+1
        IF (Big.LT.ABS(Y(K+1))) RETURN
        F0=F1
        F1=F2
        F2=F3
        F3=F(T(K+1),Y(K+1))
20    CONTINUE
      RETURN
      END

      SUBROUTINE XMILNE(F,A,B,Y0,M,T,Y,Mend)
      PARAMETER(Big=1E13)
      INTEGER J,K,M
      REAL A,B,H,F1,F2,F3,F4,Pmod,Pnew,Pold,T,TK,Y,YK,Yold,Y0
      REAL K1,K2,K3,K4,TJ,YJ
      DIMENSION T(1:1000),Y(1:1000)
      EXTERNAL F
      H=(B-A)/M
      T(0)=A
      Y(0)=Y0
C     Use Runge-Kutta to compute three starting values.
      DO 10 J=0,2
        TJ=T(J)
        YJ=Y(J)
        K1=H*F(TJ,YJ)
        K2=H*F(TJ+H/2,YJ+0.5*K1)
        K3=H*F(TJ+H/2,YJ+0.5*K2)
        K4=H*F(TJ+H,YJ+K3)
        Y(J+1)=YJ+(K1+2*K2+2*K3+K4)/6
        T(J+1)=A+H*(J+1)
10    CONTINUE
C     Start of Procedure Milne-Simpson
      F1=F(T(1),Y(1))
      F2=F(T(2),Y(2))
      F3=F(T(3),Y(3))
      Pold=0
      Yold=0
      DO 20 K=3,M-1
        Pnew=Y(K-3)+4*H*(2*F1-F2+2*F3)/3
        Pmod=Pnew+28*(Yold-Pold)/29
        T(K+1)=A+H*(K+1)
        F4=F(T(K+1),Pmod)
        Y(K+1)=Y(K-1)+H*(F2+4*F3+F4)/3
        Mend=K+1
        IF (Big.LT.ABS(Y(K+1))) RETURN
        Pold=Pnew
        Yold=Y(K+1)
        F1=F2
        F2=F3
        F3=F(T(K+1),Y(K+1))
20    CONTINUE
      RETURN
      END

      SUBROUTINE XRKF45(F,A,B,Y0,Tol,T,Y,M,MaxM,Mend)
      PARAMETER(Big=1E13)
      INTEGER I,J,K,M,Mend
      REAL A,B,T,Tol,Y,Y0
      REAL Br,H,Err,Hmin,Hmax,K1,K2,K3,K4,K5,K6,TJ,YJ
      REAL A2,B2,A3,B3,C3,A4,B4,C4,D4,A5,B5,C5,D5,E5,S
      REAL A6,B6,C6,D6,E6,F6,R1,R3,R4,R5,R6,N1,N3,N4,N5
      REAL M1,M3,M4,M5,M6,Y1,Y2,Y3,Y4,Y5,Y6,Ygood,Ynew
      DIMENSION T(1:1000),Y(1:1000)
      EXTERNAL F
      DATA A2,B2,A3,B3,C3,A4,B4,C4,D4,A5,B5,C5,D5,E5,
     A     A6,B6,C6,D6,E6,F6,R1,R3,R4,R5,R6,N1,N3,N4,N5
     B/ 0.25,   0.25,   0.375,   0.09375,  0.28125,   0.9230769231,
     C  0.8793809741,  -3.277196177,           3.320892126,    1.0,
     D  2.032407407,   -8.0,   7.173489279,   -0.2058966862,   0.5,
     E -0.2962962963,   2.0,  -1.381676413,    0.4529727096,  -0.275,
     F  0.002777777778,    -0.02994152047,    -0.02919989367,   0.02,
     G  0.03636363636, 0.1157407407, 0.5489278752, 0.535331384, -0.2/
      H=(B-A)/M
      Hmin=H/1024
      Hmax=H*64
      T(0)=A
      Y(0)=Y0
      T(0)=A
      J=0
      TJ=A
      Br=B-0.00001*ABS(B)
10    IF (T(J).LT.B) THEN
        IF ((T(J)+H).GT.Br) H=B-T(J)
        TJ=T(J)
        YJ=Y(J)
        Y1=YJ
        K1=H*F(TJ,Y1)
        Y2=YJ+B2*K1
          IF (Big.LT.ABS(Y2)) RETURN
        K2=H*F(TJ+A2*H,Y2)
        Y3=YJ+B3*K1+C3*K2
          IF (Big.LT.ABS(Y3)) RETURN
        K3=H*F(TJ+A3*H,Y3)
        Y4=YJ+B4*K1+C4*K2+D4*K3
          IF (Big.LT.ABS(Y4)) RETURN
        K4=H*F(TJ+A4*H,Y4)
        Y5=YJ+B5*K1+C5*K2+D5*K3+E5*K4
          IF (Big.LT.ABS(Y5)) RETURN
        K5=H*F(TJ+A5*H,Y5)
        Y6=YJ+B6*K1+C6*K2+D6*K3+E6*K4+F6*K5
          IF (Big.LT.ABS(Y6)) RETURN
        K6=H*F(TJ+A6*H,Y6)
        Err=ABS(R1*K1+R3*K3+R4*K4+R5*K5+R6*K6)
        Ynew=YJ+N1*K1+N3*K3+N4*K4+N5*K5
        Err=ABS(Err)
        IF (Err.LT.Tol.OR.H.LE.2*Hmin) THEN
          Y(J+1)=Ynew
          IF ((TJ+H).GT.Br) THEN
              T(J+1)=B
          ELSE
              T(J+1)=TJ+H
          ENDIF
          J=J+1
          TJ=T(J)
        ENDIF
        IF (Err.EQ.0) THEN
          S=0
        ELSE
          S=0.84*(Tol*H/Err)**0.25
        ENDIF
        IF (S.LT.0.75.AND.H.GT.(2*Hmin)) THEN
          H=H/2
        ENDIF
        IF (S.GT.1.50.AND.(2*H).LT.Hmax) THEN
          H=H*2
        ENDIF
        IF (Big.LT.ABS(Y(J)).OR.MaxM.EQ.J) RETURN
        Mend=J
        IF (B.GT.T(J)) THEN
          M=J+1
        ELSE
          M=J
        ENDIF
        WRITE(9,1000) T(J),Y(J)
        GOTO 10
      ENDIF
      RETURN
1000  FORMAT(5X,2F18.9)
      END

      SUBROUTINE MESSAGE(Meth,Order,State)
      INTEGER K,Meth,Order,State
      REAL Resp
      WRITE(9,*)'          Solution of the differential equation  Y` = F
     +(T,Y)'
      WRITE(9,*)' '
      WRITE(9,*)'          with the initial condition  Y(A) = Y .'
      WRITE(9,*)'                                              0'
      WRITE(9,*)' '
      WRITE(9,*)'          A numerical approximation is computed over [A
     +,B].'
      WRITE(9,*)' '
      WRITE(9,*)' '
      WRITE(9,*)'          Choose the method of approximation:'
      WRITE(9,*)' '
      WRITE(9,*)'    < 1 > Adams-Bashforth-Moulton method'
      WRITE(9,*)' '
      WRITE(9,*)'    < 2 > Milne-Simpson`s method'
      WRITE(9,*)' '
      WRITE(9,*)'    < 3 > Hamming`s method'
      WRITE(9,*)' '
      WRITE(9,*)'    < 4 > Taylor`s method of order N=3'
      WRITE(9,*)' '
      WRITE(9,*)'    < 5 > Taylor`s method of order N=4'
      WRITE(9,*)' '
      WRITE(9,*)'    < 6 > Runge-Kutta-Fehlberg'
      WRITE(9,*)' '
      WRITE(9,*)'          SELECT < 1 - 6 > ? '
      Resp=Meth
      READ(9,*) Resp
      Meth=INT(Resp)
      IF ((Meth.LT.1.OR.Meth.GT.6).AND.State.NE.0) THEN
        Meth=1
      ENDIF
      IF (Meth.EQ.4) THEN
        Order=3
      ENDIF
      IF (Meth.EQ.5) THEN
        Order=4
      ENDIF
      RETURN
      END

      SUBROUTINE INPUT(Meth)
      INTEGER Meth
      REAL Valu
      CHARACTER Ans*1
      DO 10 I=1,15
        WRITE(9,*)' '
10    CONTINUE
      WRITE(9,*)'     You chose '
      WRITE(9,*)' '
      IF (Meth.EQ.1) THEN
        WRITE(9,*)'     Adams-Bashforth-Moulton method to solve
     + the D.E.'
      ELSEIF (Meth.EQ.2) THEN
        WRITE(9,*)'     the Milne-Simpson method to solve
     + the D.E.'
      ELSEIF (Meth.EQ.3) THEN
        WRITE(9,*)'     Hamming`s method to solve the D.E.'
      ELSEIF (Meth.EQ.4) THEN
        WRITE(9,*)'     Taylor`s method of order N=3 to solve
     + the D.E.'
      ELSEIF (Meth.EQ.5) THEN
        WRITE(9,*)'     Taylor`s method of order N=4 to solve
     + the D.E.'
      ELSEIF (Meth.EQ.6) THEN
        WRITE(9,*)'     the Runge-Kutta-Fehlberg method to solve
     + the D.E.'
      ENDIF
      WRITE(9,*)' '
      CALL PRINTFUN
      WRITE(9,*)' '
      WRITE(9,*)'     with the initial condition Y(A) = Y0.'
      WRITE(9,*)' '
      WRITE(9,*)'     A numerical approximation is computed over [A,B].'
      WRITE(9,*)' '
      WRITE(9,*)'     You must enter the endpoints for the interval,'
      WRITE(9,*)' '
      WRITE(9,*)'     the initial condition Y0, and the number of steps
     + M.'
      WRITE(9,*)' '
      WRITE(9,*)' '
      WRITE(9,*)'         Press the <ENTER> key. '
      READ(9,'(A)') Ans
      RETURN
      END

      SUBROUTINE EPOINTS(A,B,Y0,M,Tol,State)
      INTEGER I,M,Stat,State
      REAL A,B,Tol,Y0,Valu
C     STATUS = (Change,Enter,Done)
      CHARACTER Ans*1
      Stat=1
      IF (State.EQ.0) THEN
        Stat=0
      ENDIF
10    CONTINUE
      DO 20 I=1,15
        WRITE(9,*)' '
20    CONTINUE
      CALL PRINTFUN
      WRITE(9,*)' '
      WRITE(9,*)' '
      IF (Stat.EQ.1) THEN
        WRITE(9,*)'     ENTER  the  left  endpoint  A = '
        READ(9,*) A
        WRITE(9,*)' '
        WRITE(9,*)'     ENTER  the  right endpoint  B = '
        READ(9,*) B
        WRITE(9,*)' '
        WRITE(9,*)'     ENTER initial condition  Y(A) = '
        READ(9,*) Y0
        WRITE(9,*)' '
        WRITE(9,*)'     ENTER the number of steps   M = '
        Valu=M
        READ(9,*) Valu
        M=INT(Valu)
        IF (M.LT.1) THEN
          M=1
        ENDIF
        IF (M.GT.1000) THEN
          M=1000
        ENDIF
        WRITE(9,*)'     ENTER   the   tolerance   Tol = '
        Valu=Tol
        READ(9,*) Valu
        Tol=Valu
      ELSE
        WRITE(9,*)'     The left  endpoint is    A =',A
        WRITE(9,*)' '
        WRITE(9,*)'     The right endpoint is    B =',B
        WRITE(9,*)' '
        WRITE(9,*)'     Initial condition is  Y(A) =',Y0
        WRITE(9,*)' '
        WRITE(9,*)'     The number of steps is   M = ',M
        WRITE(9,*)' '
        WRITE(9,*)'     The   tolerance  is    Tol = ',Tol
      ENDIF
      WRITE(9,*)' '
      WRITE(9,*)' '
      WRITE(9,*)'     Do you want to make a change ?  <Y/N>  '
      READ(9,'(A)') Ans
      IF (Ans.EQ.'Y'.OR.Ans.EQ.'y') THEN
          Stat=0
          DO 30 I=1,15
            WRITE(9,*)' '
30        CONTINUE
          WRITE(9,*)'          '
          CALL PRINTFUN
          WRITE(9,*)' '
          WRITE(9,*)' '
          WRITE(9,*)'     The current left  endpoint is A =',A
            WRITE(9,*)'     ENTER  the NEW left  endpoint A = '
          READ(9,*) A
          WRITE(9,*)' '
          WRITE(9,*)'     The current right endpoint is B =',B
            WRITE(9,*)'     ENTER  the NEW right endpoint B = '
          READ(9,*) B
          WRITE(9,*)' '
          WRITE(9,*)'        The current   I. C. is  Y(A) =',Y0
            WRITE(9,*)'        ENTER the NEW    I. C.  Y(A) = '
          READ(9,*) Y0
          WRITE(9,*)' '
          WRITE(9,*)'        The current value of M is  M = ',M
            WRITE(9,*)'        ENTER  the  NEW  value of  M = '
          Valu=M
          READ(9,*) Valu
          M=INT(Valu)
          IF (M.LT.1) THEN
            M=1
          ENDIF
          IF (M.GT.1000) THEN
            M=1000
          ENDIF
          WRITE(9,*)'    The current value of Tol is  Tol = ',Tol
            WRITE(9,*)'        ENTER  the NEW value of  Tol = '
          READ(9,*) Valu
          Tol=Valu
      ELSE
        Stat=2
      ENDIF
      IF (Stat.EQ.0.OR.Stat.EQ.1) GOTO 10
      RETURN
      END

      SUBROUTINE RESULTS(T,Y,M,Mend,Meth)
      INTEGER I,K,M,Mend,Meth
      REAL T,Y
      DIMENSION T(1:1000),Y(1:1000)
      DO 10 I=1,10
        WRITE(9,*)' '
10    CONTINUE
      IF (Meth.EQ.1) THEN
        WRITE(9,*)'Adams-Bashforth-Moulton method was used to solve the
     & D.E.'
      ELSEIF (Meth.EQ.2) THEN
        WRITE(9,*)'The Milne-Simpson method was used to solve the D.E.'
      ELSEIF (Meth.EQ.3) THEN
        WRITE(9,*)'Hamming`s method was used to solve the D.E.'
      ELSEIF (Meth.EQ.4) THEN
        WRITE(9,*)'Taylor`s method of order N=3 was used to solve the
     & D.E.'
      ELSEIF (Meth.EQ.5) THEN
        WRITE(9,*)'Taylor`s method of order N=4 was used to solve the
     & D.E.'
      ELSEIF (Meth.EQ.6) THEN
        WRITE(9,*)'The Runge-Kutta-Fehlberg method to solve the D.E.'
      ENDIF
      WRITE(9,*)' '
      CALL PRINTFUN
      WRITE(9,*)' '
      WRITE(9,*)'with  Y(',T(0),'  ) =',Y(0)
      WRITE(9,*)' '
      WRITE(9,*)'   K            T(K)           Y(K)'
      WRITE(9,*)' ---------------------------------------'
      DO 20 K=0,Mend
        WRITE(9,999) K, T(K), Y(K)
20    CONTINUE
      IF (Mend.LT.M) THEN
        WRITE(9,*)' '
        WRITE(9,*)'The solution points are approaching a pole.'
      ENDIF
999   FORMAT(I4,4X,4F15.7)
      PAUSE
      RETURN
      END
