      PROGRAM BDDVALUE
C     ----------------------------------------------------------------
C     Alg9'9-10.for  FORTRAN program for implementing Algorithm 9.9-10
C     
C     NUMERICAL METHODS: FORTRAN Programs, (c) John H. Mathews 1995
C     To accompany the text:
C     NUMERICAL METHODS for Math., Science & Engineering, 2nd Ed, 1992
C     Prentice Hall, Englewood Cliffs, New Jersey, 07632, U.S.A.
C     Prentice Hall, Inc.; USA, Canada, Mexico ISBN 0-13-624990-6
C     Prentice Hall, International Editions:   ISBN 0-13-625047-5
C     This free software is compliments of the author.
C     E-mail address:       in%"mathews@fullerton.edu"
C     
C     Algorithm 9.9  (Linear Shooting Method).
C     Section   9.8,  Boundary Value Problems, Page 488
C     
C     Algorithm 9.10 (Finite-Difference Method). 
C     Section   9.9,  Finite-Difference Method, Page 496
C     ----------------------------------------------------------------
      PARAMETER(GNmax = 630,MaxM = 101)
      CHARACTER Ans*1,Ach*1,Mess*80
      REAL D,T,X,X1,X2
      DIMENSION D(1:4),T(0:MaxM),X(0:MaxM),X1(0:MaxM),X2(0:MaxM)
      INTEGER FunM,FunT, GNpts, Inum, M, Mend, Meth, Order, Sub
      REAL A, B, Alpha, Beta
      INTEGER State,DoMo,Changes,Done,Working,Go,Stop
      EXTERNAL F1,F2,P,Q,R
      FunM = 9
      Meth = 1
      CALL MESSAGE(Meth)
      CALL INPUT(Meth,FunT,FunM)
      CALL EPOINTS(A,B,Alpha,Beta,M)
      IF Meth.EQ.1 THEN
        CALL LinSho(A,B,Alpha,Beta,M,T,X,FunT)
      ENDIF
      IF Meth.EQ.2 THEN
        CALL FinDif(A,B,Alpha,Beta,M,T,X,FunT)
      ENDIF
      CALL RESULTS(FunT,Meth,T,X,M,Mend)
      WRITE(9,*)' '
      READ(9,*) Ans
      STOP
      END !End of the Main Program 
  
      REAL FUNCTION P(T,FunT)
      INTEGER FunT
      REAL T
      IF (FunT.EQ.1) THEN
        P = 2.0*T/(1.0 + T*T)
      ELSEIF (FunT.EQ.2) THEN
        P = -2.0/T
      ELSEIF (FunT.EQ.3) THEN
        P = -5.0
      ELSEIF (FunT.EQ.4) THEN
        P = -2.0
      ELSEIF (FunT.EQ.5) THEN
        P = -1.0/T
      ELSEIF (FunT.EQ.6) THEN
        P = -2.0/T
      ELSEIF (FunT.EQ.7) THEN
        P = -1.0/T
      ELSEIF (FunT.EQ.8) THEN
        P = 1.0/T
      ELSEIF (FunT.EQ.9) THEN
        P = -1.0/T
      ENDIF
      RETURN
      END

      REAL FUNCTION Q(T,FunT)
      INTEGER FunT
      REAL T
      IF (FunT.EQ.1) THEN
        Q = -2.0/(1.0 + T*T)
      ELSEIF (FunT.EQ.2) THEN
        Q = 2.0/(T*T)
      ELSEIF (FunT.EQ.3) THEN
        Q = -6.0
      ELSEIF (FunT.EQ.4) THEN
        Q = -2.0
      ELSEIF (FunT.EQ.5) THEN
        Q = -(1.0 - 1.0/(4.0*T*T))
      ELSEIF (FunT.EQ.6) THEN
        Q = 2.0/(T*T)
      ELSEIF (FunT.EQ.7) THEN
        Q = -(1.0 - 1.0/(4.0*T*T))
      ELSEIF (FunT.EQ.8) THEN
        Q = -1.0/(T*T)
      ELSEIF (FunT.EQ.9) THEN
        Q = -16.0/(T*T)
      ENDIF
      RETURN
      END

      REAL FUNCTION R(T,FunT)
      INTEGER FunT
      REAL T
      IF (FunT.EQ.1) THEN
        R = 1.0
      ELSEIF (FunT.EQ.2) THEN
        R = 10.0*COS(LOG(T))/(T*T)
      ELSEIF (FunT.EQ.3) THEN
        R = T*EXP(-2.0*T) + 3.9*COS(3.0*T)
      ELSEIF (FunT.EQ.4) THEN
        R = EXP(-T) + SIN(2.0*T)
      ELSEIF (FunT.EQ.5) THEN
        R = 0.0
      ELSEIF (FunT.EQ.6) THEN
        R = SIN(T)/(T*T)
      ELSEIF (FunT.EQ.7) THEN
        R = SQRT(T)*COS(T)
      ELSEIF (FunT.EQ.8) THEN
        R = 1.0
      ELSEIF (FunT.EQ.9) THEN
        R = 1.0/(T*T)
      ENDIF
      RETURN
      END

      REAL FUNCTION F1(T,X,Y,FunT)
      INTEGER FunT
      REAL T,X,Y
      IF (FunT.EQ.1) THEN
        F1 = 2.0*T*Y/(1.0 + T*T) - 2.0*X/(1.0 + T*T) + 1.0
      ELSEIF (FunT.EQ.2) THEN
        F1 = -2.0/T*Y + 2.0/(T*T)*X + 10.0*COS(LOG(T))/(T*T)
      ELSEIF (FunT.EQ.3) THEN
        F1 = -5.0*Y - 6.0*X + T*EXP(-2.0*T) + 3.9*COS(3.0*T)
      ELSEIF (FunT.EQ.4) THEN
        F1 = -2.0*Y - 2.0*X + EXP(-T) + SIN(2.0*T)
      ELSEIF (FunT.EQ.5) THEN
        F1 = -1.0/T*Y - (1.0 - 1.0/(4.0*T*T))*X
      ELSEIF (FunT.EQ.6) THEN
        F1 = -2.0/T*Y + 2.0/(T*T)*X + SIN(T)/(T*T)
      ELSEIF (FunT.EQ.7) THEN
        F1 = -1.0/T*Y-(1.0 - 1.0/(4.0*T*T))*X+SQRT(T)*COS(T)
      ELSEIF (FunT.EQ.8) THEN
        F1 = 1.0/T*Y - 1.0/(T*T)*X + 1.0
      ELSEIF (FunT.EQ.9) THEN
        F1 = -1.0/T*Y - 16.0/(T*T)*X + 1.0/(T*T)
      ENDIF
      RETURN
      END

      REAL FUNCTION F2(T,X,Y,FunT)
      INTEGER FunT
      REAL T,X,Y
      IF (FunT.EQ.1) THEN
        F2 = 2.0*T*Y/(1.0 + T*T) - 2.0*X/(1.0 + T*T)
      ELSEIF (FunT.EQ.2) THEN
        F2 = -2.0/T*Y + 2.0/(T*T)*X
      ELSEIF (FunT.EQ.3) THEN
        F2 = -5.0*Y - 6.0*X
      ELSEIF (FunT.EQ.4) THEN
        F2 = -2.0*Y - 2.0*X
      ELSEIF (FunT.EQ.5) THEN
        F2 = -1.0/T*Y - (1.0 - 1.0/(4.0*T*T))*X
      ELSEIF (FunT.EQ.6) THEN
        F2 = -2.0/T*Y + 2.0/(T*T)*X
      ELSEIF (FunT.EQ.7) THEN
        F2 = -1.0/T*Y - (1.0 - 1.0/(4.0*T*T))*X
      ELSEIF (FunT.EQ.8) THEN
        F2 = 1.0/T*Y - 1.0/(T*T)*X
      ELSEIF (FunT.EQ.9) THEN
        F2 = -1.0/T*Y - 16/(T*T)*X
      ENDIF
      RETURN
      END

      SUBROUTINE PRTFUN(FunT)
      INTEGER FunT
      IF (FunT.EQ.1) THEN
        WRITE(9,*)'X`` =  2*T*X`/(1 + T*T) - 2*X/(1 + T*T) + 1'
      ELSEIF (FunT.EQ.2) THEN
        WRITE(9,*)'X`` =  -2/T*X` + 2/(T*T)*X + 10*COS(LOG(T))/(T*T)'
      ELSEIF (FunT.EQ.3) THEN
        WRITE(9,*)'X`` =  -5*X` - 6*X + T*EXP(-2*T) + 3.9*COS(3*T)'
      ELSEIF (FunT.EQ.4) THEN
        WRITE(9,*)'X`` =  -2*X` - 2*X + EXP(-T) + SIN(2*T)'
      ELSEIF (FunT.EQ.5) THEN
        WRITE(9,*)'X`` =  -1/T*X` - (1 - 1/(4*T*T))*X'
      ELSEIF (FunT.EQ.6) THEN
        WRITE(9,*)'X`` =  -2/T*X` + 2/(T*T)*X + SIN(T)/(T*T)'
      ELSEIF (FunT.EQ.7) THEN
        WRITE(9,*)'X`` =  -1/T*X` - (1 - 1/(4*T*T))*X + SQRT(T)*COS(T)'
      ELSEIF (FunT.EQ.8) THEN
        WRITE(9,*)'X`` =  1/T*X` - 1/(T*T)*X + 1'
      ELSEIF (FunT.EQ.9) THEN
        WRITE(9,*)'X`` =   -1/T*X` - 16/(T*T)*X + 1/(T*T)'
      ENDIF
      RETURN
      END

      !Start of the subroutine for the Linear Shooting method.
      SUBROUTINE RKbdF(F,A,B,Alpha,Beta,M,T,X,FunT)
      PARAMETER(MaxM = 101)
      INTEGER J,M,FunT
      REAL A,B,Alpha,Beta,T,X,Y
      REAL H,S1,S2 S3,S4,R1,R2,R3,R4,Tj,Xj,Yj
      DIMENSION T(0:MaxM),X(0:MaxM),Y(0:MaxM)
      EXTERNAL F
      H = (B - A)/M
      T(0) = A
      X(0) = Alpha
      Y(0) = Beta
      DO J=0,(M-1)
        Tj = T(J)
        Xj = X(J)
        Yj = Y(J)
        S1 = H*Yj
        R1 = H*F(Tj, Xj, Yj,FunT)
        S2 = H*(Yj + R1/2.0)
        R2 = H*F(Tj + H/2.0, Xj+S1/2.0, Yj+R1/2.0,FunT)
        S3 = H*(Yj + R2/2.0)
        R3 = H*F(Tj + H/2.0, Xj+S2/2.0, Yj+R2/2.0,FunT)
        S4 = H*(Yj + R3)
        R4 = H*F(Tj + H, Xj + S3, Yj + R3,FunT)
        X(J+1) = Xj + (S1 + 2.0*S2 + 2.0*S3 + S4)/6.0
        Y(J+1) = Yj + (R1 + 2.0*R2 + 2.0*R3 + R4)/6.0
        T(J+1) = A + H*(J+1)
      ENDDO
      END
     
      !Start of the main program for the Linear Shooting method.
      SUBROUTINE LinSho(A,B,Alpha,Beta,M,T,X,FunT)
      PARAMETER(MaxM = 101)
      INTEGER J,M,FunT
      REAL A,B,Alpha,Beta
      REAL T,X,X1,X2
      DIMENSION T(0:MaxM),X(0:MaxM),X1(0:MaxM),X2(0:MaxM)
      EXTERNAL F1,F2
      CALL RKbdF(F1,A,B,Alpha,0.0,M,T,X1,FunT)
      CALL RKbdF(F2,A,B, 0.0 ,1.0,M,T,X2,FunT)
      DO J=0,M
        X(J) = X1(J) + (Beta - X1(M))*X2(J)/X2(M)
      ENDDO
      END   !The main program for the Linear Shooting method.

      !Start of the subroutines for the Finite Difference method.
      SUBROUTINE CoeMat(A,B,Alpha,Beta,N,Va,Vb,Vc,Vd,Vt,FunT)
      PARAMETER(MaxM = 101)
      INTEGER J,N,FunT
      REAL A,B,Alpha,Beta
      REAL Va, Vb, Vc, Vd, Vt
      DIMENSION Va(0:MaxM),Vb(0:MaxM),Vc(0:MaxM),Vd(0:MaxM),Vt(0:MaxM)
      EXTERNAL P,Q,R
      H = (B - A)/N
      DO J=1,(N-1)
        Vt(J) = A + H*J
      ENDDO
      DO J=1,(N-1)
        Vb(J) = -H*H*R(Vt(J),FunT)
      ENDDO
      Vb(1) = Vb(1) + (1 + H/2*P(Vt(1),FunT))*Alpha
      Vb(N-1) = Vb(N-1) + (1 - H/2*P(Vt(N-1),FunT))*Beta
      DO J=1,(N-1)
        Vd(J) = 2 + H*H*Q(Vt(J),FunT)
      ENDDO
      DO J=1,(N-2)
        Va(J) = -1 - H/2*P(Vt(J+1),FunT)
      ENDDO
      DO J=1,(N-2)
        Vc(J) = -1 + H/2*P(Vt(J),FunT)
      ENDDO
      END

      SUBROUTINE TriMat(Va,Vb,Vc,Vd,N,X)
      PARAMETER(MaxM = 101)
      INTEGER K,N
      REAL Va,Vb,Vc,Vd,X
      DIMENSION Va(0:MaxM),Vb(0:MaxM),Vc(0:MaxM),Vd(0:MaxM),X(0:MaxM)
      REAL TEM
      DO K=2,(N-1)
        TEM = Va(K-1)/Vd(K-1)
        Vd(K) = Vd(K) - TEM*Vc(K-1)
        Vb(K) = Vb(K) - TEM*Vb(K-1)
      ENDDO
      X(N-1) = Vb(N-1)/Vd(N-1)
      DO K=(N-2),1,-1
        X(K) = (Vb(K) - Vc(K)*X(K+1))/Vd(K)
      ENDDO
      END

      !Start of the main program for the Finite Difference method.
      SUBROUTINE FinDif(A,B,Alpha,Beta,N,T,X,FunT)
      PARAMETER(MaxM = 101)
      INTEGER N,FunT
      REAL A,B,Alpha,Beta,H
      REAL T, X
      DIMENSION T(0:MaxM), X(0:MaxM)
      REAL Va, Vb, Vc, Vd
      DIMENSION Va(0:MaxM), Vb(0:MaxM), Vc(0:MaxM), Vd(0:MaxM)
      !Remark: Some of the zero components are NOT used.
      EXTERNAL P,Q,R
      CALL CoeMat(A,B,Alpha,Beta,N,Va,Vb,Vc,Vd,T,FunT)
      CALL TriMat(Va, Vb, Vc, Vd, N, X)
      T(0) = A
      T(N) = B
      X(0) = Alpha
      X(N) = Beta
      END   !The main program for the Finite Difference method.

      SUBROUTINE MESSAGE(Meth)
      INTEGER Meth
      CHARACTER Mess*80
      WRITE(9,*)'       BOUNDARY VALUE DIFFERENTIAL EQUATIONS'
      WRITE(9,*)' '
      WRITE(9,*)' '
      WRITE(9,*)'Solution of the differential equation'
      WRITE(9,*)' '
      WRITE(9,*)'x`` = p(t)x ` ( t ) + q ( t ) x ( t ) + r ( t )'
      WRITE(9,*)' '
      WRITE(9,*)'with x(a)=Alpha and x(b)=Beta computed over [a,b].'
      WRITE(9,*)' '
      WRITE(9,*)' '
      WRITE(9,*)'       Choose the method of approximation:'
      WRITE(9,*)' '
      WRITE(9,*)'       < 1 >  Linear shooting method'
      WRITE(9,*)' '
      WRITE(9,*)'       < 2 >  Finite difference method'
      WRITE(9,*)' '
      Mess = '       SELECT < 1 - 2 > ?  '
      Meth = 1
      WRITE(9,*) Mess
      READ(9,*)  Meth
      IF (Meth .LE. 1) .AND. (State .NE. Changes) THEN
        Meth = 1
      ENDIF
      IF (Meth .GT. 2) .AND. (State .NE. Changes) THEN
        Meth = 2
      ENDIF
      END

      SUBROUTINE INPUT(Meth,FunT,FunM)
      INTEGER FunT,FunM,K
      CHARACTER Ans*1,Mess*80
      WRITE(9,*)' '
      IF Meth.EQ.1 THEN
        WRITE(9,*)'You chose the linear shooting method'
      ENDIF  
      IF Meth.EQ.2 THEN
        WRITE(9,*)'You chose the finite difference method'
      ENDIF  
      WRITE(9,*)'to solve Y` = F(T,Y).'
      WRITE(9,*)' '
      WRITE(9,*)'Choose your D.E.:'
      WRITE(9,*)' '
      DO K=1,FunM
        WRITE(9,*)'<', K,'>'
        CALL PRTFUN(K)
        WRITE(9,*)' '
      ENDDO
      Mess = '     SELECT < 1 - 9 > ?  '
      WRITE(9,*) Mess
      READ(9,*) FunT
      IF (FunT.LT.1) THEN
        FunT = 1
      ENDIF
      IF (FunT.GT.FunM) THEN
        FunT = FunM
      ENDIF
      WRITE(9,*)' '
      IF Meth.EQ.1 THEN
        WRITE(9,*)'You chose the linear shooting method'
      ENDIF  
      IF Meth.EQ.2 THEN
        WRITE(9,*)'You chose the finite difference method'
      ENDIF  
      WRITE(9,*)'to solve the D.E.'
      WRITE(9,*)' '
      WRITE(9,*)'                     '
      CALL PRTFUN(FunT)
      WRITE(9,*)' '
      WRITE(9,*)'with the initial conditions'
      WRITE(9,*)' '
      WRITE(9,*)'X(A) = Alpha  and  X(B) = Beta.'
      WRITE(9,*)' '
      WRITE(9,*)'A numerical approximation is computed over [A,B].'
      WRITE(9,*)' '
      WRITE(9,*)'You must give the endpoints for the interval,'
      WRITE(9,*)' '
      WRITE(9,*)'the boundary values, and the number of steps.'
      WRITE(9,*)' '
      WRITE(9,*)' '
      WRITE(9,*)'     Press the <ENTER> key. '
      READ(9,*) Ans
      WRITE(9,*)' '
      END
      
      SUBROUTINE EPOINTS(A,B,Alpha,Beta,M)
      INTEGER M
      REAL A,B,Alpha,Beta
      CHARACTER Mess*80
      WRITE(9,*)' '
      WRITE(9,*)'                  '
      CALL PRTFUN(FunT)
      WRITE(9,*)' '
      WRITE(9,*)' '
      Mess = 'ENTER  the  left  endpoint  A = '
      WRITE(9,*) Mess
      READ(9,*)  A
      WRITE(9,*)' '
      Mess = 'ENTER  the  right endpoint  B = '
      WRITE(9,*) Mess
      READ(9,*)  B
      WRITE(9,*)' '
      Mess = 'ENTER initial condition  X(A) = '
      WRITE(9,*) Mess
      READ(9,*) Alpha
      WRITE(9,*)' '
      Mess = 'ENTER initial condition  X(B) = '
      WRITE(9,*) Mess
      READ(9,*)  Beta
      WRITE(9,*)' '
      Mess = 'ENTER the number of steps   M = '
      M = 1
      WRITE(9,*) Mess
      READ(9,*) M
      WRITE(9,*)' '
      IF (M.LT.1) M = 1
      IF (M.GT.1000) M = 1000
      END

      SUBROUTINE RESULTS(FunT,Meth,T,X,M,Mend)
      INTEGER FunT,K,M,Mend
      REAL T,X
      DIMENSION T(0:MaxM),X(0:MaxM)
      WRITE(9,*)' '
      IF Meth.EQ.1 THEN
        WRITE(9,*)' The linear shooting method'
      ENDIF  
      IF Meth.EQ.2 THEN
        WRITE(9,*)' The finite difference method'
      ENDIF  
      WRITE(9,*)' was used to solve the D.E.'
      WRITE(9,*)'       '
      CALL PRTFUN(FunT)
      WRITE(9,*)' '
      WRITE(9,*)'with  X(', T(0), '  ) =', X(0)
      WRITE(9,*)'and   X(', T(M), '  ) =', X(M)
      WRITE(9,*)' '
      WRITE(9,*)'    K', '         T(K)    ', '         X(K)'
      WRITE(9,*)'  --------------------------------------------'
      WRITE(9,*)' '
      DO K=0,M
        WRITE(9,*)'  ',K, '        ', T(K), '        ', X(K)
      ENDDO
      Mend = M
      IF (Mend.LT.M) THEN 
        WRITE(9,*)'The solution points are approaching a pole.'
        WRITE(9,*)' '
      ENDIF
      WRITE(9,*)'                   Press the <ENTER> key.  '
      READ(9,*) Ans
      WRITE(9,*)' '
      END
